package hu.kozadat.search.base.preprocessor;

import hu.kozadat.search.persistence.LabelCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
@Component("fieldNamePreprocessor")
public class FieldNamePreprocessor extends QueryPreprocessor {

    private LabelCache labelCache;

    private static final Pattern fieldPattern = Pattern.compile("([\\wáéűőúöüóíÁÉŰŐÚÖÜÓÍ]+):([^\" ][\\S]*|$)?", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);

    public QueryPreprocessorResult preprocess(String segment, boolean appendFieldName) {
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(segment);
        Matcher fieldMatcher = fieldPattern.matcher(segment);
        StringBuffer sb = new StringBuffer();

        while (fieldMatcher.find()) {
            String originalFieldName = com.actiwise.einfoszab.Utils.null2String(fieldMatcher.group(1)).toLowerCase();
            String fieldValue = com.actiwise.einfoszab.Utils.null2String(fieldMatcher.group(2));

            String newFieldName = labelCache.getLabelTranslation(originalFieldName);

            if (newFieldName == null) {
                newFieldName = originalFieldName;
            } else {
                newFieldName = newFieldName.trim().toLowerCase();
            }

            String replaceSegment = !"".equals(fieldValue) ?
                    newFieldName + ":" + fieldValue :
                    newFieldName + ":";

            fieldMatcher.appendReplacement(sb, replaceSegment);
        }
        fieldMatcher.appendTail(sb);

        res.setFilteredQuery(sb.toString());
        return res;
    }

    @Autowired
    public void setLabelCache(LabelCache labelCache) {
        this.labelCache = labelCache;
    }
}
