package hu.kozadat.search.base.preprocessor;

import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * User: codex
 */
public class QueryPreprocessorManager {
    public static final String QUERY_DELIMITER = "\"";

    private List<QueryPreprocessor> unquotedProcessorLst;
    private List<QueryPreprocessor> quotedProcessorLst;

    private enum ProcessMode {EVALUATE, COPY_QUOTED, EVALUATE_QUOTED}

    private static final Pattern fieldPattern = Pattern.compile("([\\wáéűőúöüóíÁÉŰŐÚÖÜÓÍ]+):$", Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE);

    public QueryPreprocessorResult preprocess(String query, boolean evaluateQuoted) {
        StringTokenizer tokenizer = new StringTokenizer(query, QUERY_DELIMITER, true);
        StringBuffer filteredQuery = new StringBuffer();
        QueryPreprocessorResult res = new QueryPreprocessorResult();
        res.setOriginalQuery(query);
        InvalidCharPreprocessor invalidPre = new InvalidCharPreprocessor();
        TextQuoterPreprocessor quoterPre = new TextQuoterPreprocessor();

        String lastToken = "";
        ProcessMode processMode = ProcessMode.EVALUATE;
        while (tokenizer.hasMoreTokens()) {
            String actToken = tokenizer.nextToken();

            if (QUERY_DELIMITER.equals(actToken)) {
                if (evaluateQuoted) {
                    switch (processMode) {
                        case EVALUATE: {
                            Matcher fieldMatcher = fieldPattern.matcher(lastToken);
                            if (fieldMatcher.find()) {
                                processMode = ProcessMode.EVALUATE_QUOTED;
                            } else {
                                processMode = ProcessMode.COPY_QUOTED;
                                filteredQuery.append(actToken);
                            }

                            break;
                        }

                        case EVALUATE_QUOTED: {
                            processMode = ProcessMode.EVALUATE;
                            break;
                        }

                        case COPY_QUOTED: {
                            processMode = ProcessMode.EVALUATE;
                            filteredQuery.append(actToken);
                            break;
                        }
                    }
                } else {
                    processMode = processMode == ProcessMode.EVALUATE ? ProcessMode.COPY_QUOTED : ProcessMode.EVALUATE;
                    filteredQuery.append(actToken);
                }
            } else {
                QueryPreprocessorResult segmentResult = invalidPre.preprocess(actToken, true);
                if (segmentResult.getRemovedChars().size() != 0)
                    res.getRemovedChars().addAll(segmentResult.getRemovedChars());

                switch (processMode) {
                    case EVALUATE: {
                        for (QueryPreprocessor actProc : unquotedProcessorLst) {
                            segmentResult = actProc.preprocess(segmentResult.getFilteredQuery(), true);
                        }
                        break;
                    }

                    case EVALUATE_QUOTED: {
                        boolean queryModified = false;

                        for (QueryPreprocessor actProc : quotedProcessorLst) {
                            segmentResult = actProc.preprocess(segmentResult.getFilteredQuery(), false);
                            if (segmentResult.isQueryModified())
                                queryModified = true;
                        }

                        if (!queryModified)
                            segmentResult = quoterPre.preprocess(segmentResult.getFilteredQuery(), false);

                        break;
                    }

                    case COPY_QUOTED: {
                        //-- there is nothing to do here, the content of the segment is copied.
                        break;
                    }
                }

                filteredQuery.append(segmentResult.getFilteredQuery());
            }

            lastToken = actToken;
        }

        res.setFilteredQuery(filteredQuery.toString());
        return res;
    }

    public void setUnquotedProcessorLst(List<QueryPreprocessor> unquotedProcessorLst) {
        this.unquotedProcessorLst = unquotedProcessorLst;
    }

    public void setQuotedProcessorLst(List<QueryPreprocessor> quotedProcessorLst) {
        this.quotedProcessorLst = quotedProcessorLst;
    }
}
