package hu.kozadat.search.base.renderer;

import hu.kozadat.search.model.ResultItem;
import hu.kozadat.search.persistence.SearchCache;
import hu.scriptum.mdc.datatype.MDcRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletResponse;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * Absztrakt osztaly, amely az osszes talalat altal hasznalt funkciokat megvalositja.
 *
 * @author Csaba Horvath, Scriptum Inc.
 * @version $Id$
 */
@Deprecated
public abstract class AbstractResultListItemRenderer implements ResultListItemRenderer {

    private static final Logger LOGGER = LoggerFactory.getLogger(AbstractResultListItemRenderer.class);

    private static final SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");

    private Map elementConfigMap = new HashMap();
    private Map qualifierConfigMap = new HashMap();
    private Map qualifierConfigOptionalMap = new HashMap();
    private SearchCache searchCache;
    private Map internalValueMap;
    private StringBuffer keywordBuf;
    private String urlPrefix;

    public void setUrlPrefix(String urlPrefix) {
        this.urlPrefix = urlPrefix;
    }

    public String getUrlPrefix() {
        return urlPrefix;
    }

    public void setElementConfig(String key, String elementName, String qualifier, boolean qualifierOptional) {
        elementConfigMap.put(key, elementName);
        qualifierConfigMap.put(key, qualifier);
        qualifierConfigOptionalMap.put(key, qualifierOptional ? Boolean.TRUE : Boolean.FALSE);
    }

    public void setElementConfig(String key, String value) {
        if (value != null && value.trim().length() > 0) {
            int sepaIndex = value.indexOf('/');
            if (sepaIndex < 0) {
                elementConfigMap.put(key, value);
                qualifierConfigMap.put(key, null);
                qualifierConfigOptionalMap.put(key, Boolean.FALSE);
            } else {
                elementConfigMap.put(key, value.substring(0, sepaIndex));
                if (value.endsWith("?")) {
                    qualifierConfigMap.put(key, value.substring(sepaIndex + 1, value.length() - 1));
                    qualifierConfigOptionalMap.put(key, Boolean.TRUE);
                } else {
                    qualifierConfigMap.put(key, value.substring(sepaIndex + 1));
                    qualifierConfigOptionalMap.put(key, Boolean.FALSE);
                }
            }
        }
    }

    public void setSearchCache(SearchCache cache) {
        searchCache = cache;
    }

    /**
     * A kulonbozo KEY_... konstansokkal lehet erteket visszakerni.
     *
     * @param key key to get data from
     * @return data stored for the specified key (can be <code>null</code>)
     */
    public String getInternalValue(String key) {
        return internalValueMap == null ? null : (String) internalValueMap.get(key);
    }

    /**
     * Minden sor feldolgozasa soran meghivodik ez a metodus, lehetoseget biztositva ezzel
     * a talalati listaban megjeleno szoveg osszeallitasahoz.
     *
     * @param key               element name
     * @param qualifier         qualifier name (if exist), empty otherwise
     * @param value             value for the key
     * @param containsHighlight if true value contains highlighted region
     */
    public abstract void processLine(String key, String qualifier, String value, boolean containsHighlight);

    /**
     * A teljes metarekord feldolgozasa utan hivjuk meg, visszaadja az osszeallitott, talalati
     * listaban megjeleno erteket.
     *
     * @param item        result item to help setting fragment
     * @param resultIndex the selected item will be in the specified index
     * @return result html fragment (will be shown in the result list)
     */
    public abstract String getResultHtmlFragment(ResultItem item, int resultIndex);

    public ResultItem getResultItemFor(String resultText, int resultIndex, HttpServletResponse response) {
        if (internalValueMap == null) {
            internalValueMap = new HashMap();
        } else {
            internalValueMap.clear();
        }
        keywordBuf = new StringBuffer(64);

        ResultItem returnValue = new ResultItem();
        MDcRecord record = new MDcRecord();

        String[] lines = resultText.split("\n");
        for (int i = 0; i < lines.length; i++) {
            String line = lines[i].trim();
            int index = line.indexOf(':');

            if (!line.startsWith("$")) {
                String element = line.substring(0, index);
                String qualifier = "";
                int perIndex = element.indexOf('/');
                if (perIndex > 0) {
                    qualifier = element.substring(perIndex + 1).trim();
                    element = element.substring(0, perIndex).trim();
                }
                String value = line.substring(index + 1).trim();
                value = value.replaceAll("<span class=\"highlight\">http</span>://", "http://");
                boolean containsHighlight = value.indexOf("<span class=\"highlight") >= 0;
                record.addItem(searchCache.getHungarianElementName(element), searchCache.getHungarianQualifierName(qualifier), processLinks(response, value));
                processLine(element, qualifier, value, containsHighlight);
                for (Iterator iter = elementConfigMap.keySet().iterator(); iter.hasNext(); ) {
                    String key = (String) iter.next();

                    String keyElement = (String) elementConfigMap.get(key);
                    if (keyElement != null && keyElement.equals(element)) {
                        String keyQual = (String) qualifierConfigMap.get(key);
                        boolean qualifierOptional = ((Boolean) qualifierConfigOptionalMap.get(key)).booleanValue();
                        boolean needToInsert = false;
                        if (keyQual != null && keyQual.equals(qualifier)) {
                            needToInsert = true;
                        } else if (internalValueMap.get(key) == null && (keyQual == null || qualifierOptional)) {
                            needToInsert = true;
                        }

                        if (needToInsert) {
                            if ("url".equals(key)) {
                                internalValueMap.put(key, containsHighlight ? SearchCache.stripHtmlTags(value) : value);
                            } else if ("keyword".equals(key)) {
                                if (keywordBuf.length() > 0) {
                                    keywordBuf.append(", ");
                                }
                                keywordBuf.append(value);
                            } else {
                                internalValueMap.put(key, value);
                            }
                            if ("title".equals(key) && returnValue.getTitle() == null) {
                                returnValue.setTitle(SearchCache.stripHtmlTags(value));
                            }
                        }
                    }
                }
            } else {
                index = line.indexOf('=');
                String key = line.substring(0, index);
                String value = line.substring(index + 1).trim();
                if ("$record_id".equals(key)) {
                    returnValue.setId(value);
                } else if ("$provider_name".equals(key)) {
                    returnValue.setDataProvider(value);
                } else if ("$provider_id".equals(key)) {
                    returnValue.setDataProviderId(Long.parseLong(value));
                } else if ("$schema_name".equals(key)) {
                    returnValue.setMetadataType(value);
                } else if ("$modified".equals(key)) {
                    Date modified = null;

                    try {
                        modified = sdf.parse(value);
                    } catch (ParseException pex) {
                    }

                    returnValue.setModificationDate(modified);
                }
                internalValueMap.put(key, value);
            }
        }
        if (keywordBuf.length() > 0) {
            internalValueMap.put("keyword", keywordBuf.toString());
        }
        returnValue.setRecord(record);
        String resultHtmlFragment = getResultHtmlFragment(returnValue, resultIndex);
        returnValue.setResultHtmlFragment(resultHtmlFragment);

        return returnValue;
    }

    private String processLinks(HttpServletResponse response, String value) {
        StringBuffer buf = new StringBuffer(value.length() + 64);
        int prevIndex = 0;
        int strIndex = getNextLink(value, prevIndex);
        while (strIndex >= 0) {
            if (prevIndex < strIndex) {
                buf.append(value.substring(prevIndex, strIndex));
            }
            int endIndex = -1;
            if (strIndex > 0) {
                char prev = value.charAt(strIndex - 1);
                if (prev == '(') {
                    endIndex = value.indexOf(")", strIndex);
                } else if (prev == '"') {
                    endIndex = value.indexOf("\"", strIndex);
                } else {
                    endIndex = value.indexOf(" ", strIndex);
                }
            } else {
                endIndex = value.indexOf(" ", strIndex);
            }

            if (endIndex >= 0) {
                String link = value.substring(strIndex, endIndex);
                String subLink = link;
                while (link.endsWith("<span")) {
                    int endSpanIndex = value.indexOf("</span>", endIndex);
                    if (strIndex > 0) {
                        char prev = value.charAt(strIndex - 1);
                        if (prev == '(') {
                            endIndex = value.indexOf(")", endSpanIndex);
                        } else if (prev == '"') {
                            endIndex = value.indexOf("\"", endSpanIndex);
                        } else {
                            endIndex = value.indexOf(" ", endSpanIndex);
                        }
                    } else {
                        endIndex = value.indexOf(" ", endSpanIndex);
                    }

                    if (endIndex >= 0) {
                        link = value.substring(strIndex, endIndex);
                    } else {
                        link = value.substring(strIndex);
                    }
                    subLink = SearchCache.stripHtmlTags(link);
                }
                buf.append("<a href=\"");
                // buf.append(response.encodeRedirectURL(subLink));
                buf.append(subLink);
                buf.append("\">");
                buf.append(link);
                buf.append("</a>");

                if (endIndex >= 0) {
                    prevIndex = endIndex;
                } else {
                    prevIndex = value.length();
                }
            } else {
                String link = value.substring(strIndex);
                buf.append("<a href=\"");
                //buf.append(response.encodeRedirectURL(link));
                buf.append(link);
                buf.append("\">");
                buf.append(link);
                buf.append("</a>");
                prevIndex = value.length();
            }
            strIndex = getNextLink(value, prevIndex);
        }
        if (prevIndex < value.length()) {
            buf.append(value.substring(prevIndex));
        }

        return buf.toString();
    }

    private int getNextLink(String value, int position) {
        int returnValue = Integer.MAX_VALUE;

        int http = value.indexOf("http://", position);
        if (http >= 0 && returnValue > http) {
            returnValue = http;
        }
        int https = value.indexOf("https://", position);
        if (https >= 0 && returnValue > https) {
            returnValue = https;
        }
        int ftp = value.indexOf("ftp://", position);
        if (ftp >= 0 && returnValue > ftp) {
            returnValue = ftp;
        }
        int mms = value.indexOf("mms://", position);
        if (mms >= 0 && returnValue > mms) {
            returnValue = mms;
        }
        int mailTo = value.indexOf("mailto:", position);
        if (mailTo >= 0 && returnValue > mailTo) {
            returnValue = mailTo;
        }
        if (returnValue == Integer.MAX_VALUE) {
            returnValue = -1;
        }

        return returnValue;
    }
}
