package hu.kozadat.search.persistence;

import org.apache.commons.configuration.DatabaseConfiguration;

import javax.sql.DataSource;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2012.03.06.
 * Time: 10:48
 * To change this template use File | Settings | File Templates.
 */
public class ApplicationConfiguration {

    private DataSource dataSource;
    
    private String configTableName;
    private String configNameColumnName;
    private String configKeyColumnName;
    private String configValueColumnName;
    
    private Map<String, DatabaseConfiguration> configurationGroups = new TreeMap();

    public ApplicationConfiguration(DataSource dataSource, String configTableName, String configNameColumnName, String configKeyColumnName, String configValueColumnName) {
        if (dataSource == null) {
            throw new IllegalArgumentException("dataSource cannot be null");
        }
        if (configTableName == null) {
            throw new IllegalArgumentException("configTableName cannot be null");
        }
        if (configNameColumnName == null) {
            throw new IllegalArgumentException("configNameColumnName cannot be null");
        }
        if (configKeyColumnName == null) {
            throw new IllegalArgumentException("configKeyColumnName cannot be null");
        }
        if (configValueColumnName == null) {
            throw new IllegalArgumentException("configValueColumnName cannot be null");
        }
        
        this.dataSource = dataSource;
        this.configTableName = configTableName;
        this.configNameColumnName = configNameColumnName;
        this.configKeyColumnName = configKeyColumnName;
        this.configValueColumnName = configValueColumnName;
    }
    
    public void addGroup(String configurationGroupName) {
        configurationGroups.put(configurationGroupName, new DatabaseConfiguration(dataSource, configTableName, configNameColumnName, configKeyColumnName, configValueColumnName, configurationGroupName));
    }
    
    public String getStringValue(String configurationGroupName, String configurationKey) {
        return getStringValue(configurationGroupName, configurationKey, null);
    }

    public String getStringValue(String configurationGroupName, String configurationKey, String defaultValue) {
        DatabaseConfiguration configuration = configurationGroups.get(configurationGroupName);
        if (configuration == null) {
            return null;
        }
        return configuration.getString(configurationKey, defaultValue);
    }

    public Integer getIntegerValue(String configurationGroupName, String configurationKey) {
        return getIntegerValue(configurationGroupName, configurationKey, null);
    }

    public Integer getIntegerValue(String configurationGroupName, String configurationKey, Integer defaultValue) {
        DatabaseConfiguration configuration = configurationGroups.get(configurationGroupName);
        if (configuration == null) {
            return null;
        }
        return configuration.getInteger(configurationKey, defaultValue);
    }

    public Long getLongValue(String configurationGroupName, String configurationKey) {
        return getLongValue(configurationGroupName, configurationKey, null);
    }

    public Long getLongValue(String configurationGroupName, String configurationKey, Long defaultValue) {
        DatabaseConfiguration configuration = configurationGroups.get(configurationGroupName);
        if (configuration == null) {
            return null;
        }
        return configuration.getLong(configurationKey, defaultValue);
    }

    public Boolean getBooleanValue(String configurationGroupName, String configurationKey) {
        return getBooleanValue(configurationGroupName, configurationKey, null);
    }

    public Boolean getBooleanValue(String configurationGroupName, String configurationKey, Boolean defaultValue) {
        DatabaseConfiguration configuration = configurationGroups.get(configurationGroupName);
        if (configuration == null) {
            return null;
        }
        return configuration.getBoolean(configurationKey, defaultValue);
    }

    public Map<String, String> getGroupEntries(String configurationGroupName) {
        DatabaseConfiguration configuration = configurationGroups.get(configurationGroupName);
        Map<String, String> configurationGroupEntries = new TreeMap<String, String>();
        if (configuration == null) {
            return configurationGroupEntries;
        }
        Iterator<String> keys = configuration.getKeys();
        while (keys.hasNext()) {
            String key = keys.next();
            String value = configuration.getString(key);
            configurationGroupEntries.put(key, value);
        }
        return configurationGroupEntries;
    }
}
