package hu.kozadat.search.persistence.core;

import hu.kozadat.search.model.DataProviderDisplayInfo;
import hu.scriptum.einfoszab.datamodel.*;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * User: codex
 */
public class DataProviderDisplayInfoDAO
{

	public static final String TYPE_NAME_ACTIVE_DATAPROVIDER = "ActiveDataProvider";
	public static final String TYPE_NAME_REGISTERED_DATAPROVIDER = "RegisteredDataProvider";
	public static final String TYPE_NAME_EXTERNAL_INSTITUTE = "ExternalInstitute";

	private static final String FIELD_SELECTOR = ".";

	private static final String TABLE_DATA_PROVIDERS = "data_providers";
	private static final String TABLE_ALIAS_DATA_PROVIDERS = "dp";
	private static final String COLUMN_DATA_PROVIDERS_ID = "id";
	private static final String COLUMN_DATA_PROVIDERS_NAME = "name";
	private static final String COLUMN_DATA_PROVIDERS_SHORT_NAME = "short_name";
	private static final String COLUMN_DATA_PROVIDERS_ADDRESS = "address";
	private static final String COLUMN_DATA_PROVIDERS_MAIL_ADDRESS = "mail_address";
	private static final String COLUMN_DATA_PROVIDERS_WEB = "web";
	private static final String COLUMN_DATA_PROVIDERS_EMAIL = "email";
	private static final String COLUMN_DATA_PROVIDERS_LEADER = "leader";
	private static final String COLUMN_DATA_PROVIDERS_PHONE_NUM = "phone_num";
	private static final String COLUMN_DATA_PROVIDERS_FAX_NUM = "fax_num";
	private static final String COLUMN_DATA_PROVIDERS_OAI_ID = "oai_id";
	private static final String COLUMN_DATA_PROVIDERS_USER_TYPE_ID = "user_type_id";
	private static final String COLUMN_DATA_PROVIDERS_STATE = "state";

	private static final String TABLE_USER_TYPES = "user_types";
	private static final String TABLE_ALIAS_USER_TYPES = "ut";
	private static final String COLUMN_USER_TYPES_ID = "id";
	private static final String COLUMN_USER_TYPES_USER_TYPE = "user_type";

	private static final String TABLE_DP_PHONE = "dp_phone";
	private static final String COLUMN_DP_PHONE_PHONE_NUM = "phone_num";
	private static final String COLUMN_DP_PHONE_ID = "id";
	private static final String COLUMN_DP_PHONE_DP_ID = "dp_id";
	private static final String COLUMN_DP_PHONE_IS_FAX = "is_fax";


	private static final String QUERY_INSTITUTE_BY_NAME =
			"SELECT "
				+ COLUMN_DATA_PROVIDERS_ID +
			" FROM "
				+ TABLE_DATA_PROVIDERS +
			" WHERE "
				+ COLUMN_DATA_PROVIDERS_NAME + " = ?";

	private static final String QUERY_INSTITUTE_BY_ID =
			"SELECT "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_NAME + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_SHORT_NAME + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_ADDRESS + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_MAIL_ADDRESS + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_WEB + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_EMAIL + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_LEADER + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_PHONE_NUM + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_FAX_NUM + ", "
				+ TABLE_ALIAS_USER_TYPES + FIELD_SELECTOR + COLUMN_USER_TYPES_USER_TYPE + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_OAI_ID + ", "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_ID + ", "
                + TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_STATE +
			" FROM "
				+ TABLE_DATA_PROVIDERS + " " + TABLE_ALIAS_DATA_PROVIDERS + ", "
				+ TABLE_USER_TYPES + " " + TABLE_ALIAS_USER_TYPES +
			" WHERE "
				+ TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_ID + "=?"
				+ " AND " + TABLE_ALIAS_DATA_PROVIDERS + FIELD_SELECTOR + COLUMN_DATA_PROVIDERS_USER_TYPE_ID + " = " + TABLE_ALIAS_USER_TYPES + FIELD_SELECTOR + COLUMN_USER_TYPES_ID;

	private static final String QUERY_FULL_ADDRESS =
			"select "
					+ "a.id" + " as addressId, "
					+ "a.zipcode_settlement_id" + ", "
					+ "a.address" + " as addressAddress, "
					+ "a.latitude" + " as addressLatitude, "
					+ "a.longitude" + " as addressLongitude, "
					+ "z.id" + " as zipCodeId, "
					+ "z.zipcode" + ", "
					+ "s.id" + " as " + "settlementId" + ", "
					+ "s.name" + " as " + "settlementName" + ", "
					+ "s.latitude" + " as settlementLatitude, "
					+ "s.longitude" + " as settlementLongitude, "
					+ "sr.id" + " as subRegionId, "
					+ "sr.name" + " as subRegionName, "
					+ "sr.latitude" + " as subRegionLatitude, "
					+ "sr.longitude" + " as subRegionLongitude, "
					+ "c.id" + " as countyId, "
					+ "c.name" + " as countyName, "
					+ "c.latitude" + " as countyLatitude, "
					+ "c.longitude" + " as countyLongitude, "
					+ "r.id" + " as regionId, "
					+ "r.name" + " as regionName, "
					+ "r.latitude" + " as regionLatitude, "
					+ "r.longitude" + " as regionLongitude"
					+ " from "
					+ "data_providers dp "
					+ "left join " + "dp_address" + " a " + "on " + "dp.address_id = a.id "
					+ "left join " + "zipcode_settlement" + " z " + "on a.zipcode_settlement_id = z.id "
					+ "left join " + "settlement" + " s " + "on z.settlement_id = s.id "
					+ "left join " + "subregion" + " sr " + "on s.subregion_id = sr.id "
					+ "left join " + "county" + " c " + "on sr.county_id = c.id "
					+ "left join " + "region" + " r " + "on c.region_id = r.id"
					+ " where "
					+ "dp.id=?";


	// institutes
	public static final String TABLE_EXTERNAL_INSTITUTE = "external_institute";
	public static final String TABLE_ALIAS_EXTERNAL_INSTITUTE = "ei";
	public static final String COLUMN_EXTERNAL_INSTITUTE_ID = "id";
	public static final String COLUMN_EXTERNAL_INSTITUTE_NAME = "name";
	public static final String COLUMN_EXTERNAL_INSTITUTE_ADDRESS = "address";
	public static final String COLUMN_EXTERNAL_INSTITUTE_MAIL_ADDRESS = "mail_address";
	public static final String COLUMN_EXTERNAL_INSTITUTE_WEB = "web";
	public static final String COLUMN_EXTERNAL_INSTITUTE_EMAIL = "email";
	public static final String COLUMN_EXTERNAL_INSTITUTE_LEADER = "leader";

	// phones
	public static final String TABLE_EI_PHONE = "ei_phone";
	public static final String TABLE_ALIAS_EI_PHONE = "eip";
	public static final String COLUMN_EI_PHONE_ID = "id";
	public static final String COLUMN_EI_PHONE_EI_ID = "ei_id";
	public static final String COLUMN_EI_PHONE_PHONE_NUM = "phone_num";
	public static final String COLUMN_EI_PHONE_IS_FAX = "is_fax";

	private static final String QUERY_EXTERNAL_INSTITUTE_BY_ID =
			"SELECT "
				+ COLUMN_EXTERNAL_INSTITUTE_ID + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_NAME + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_ADDRESS + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_MAIL_ADDRESS + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_WEB + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_EMAIL + ", "
				+ COLUMN_EXTERNAL_INSTITUTE_LEADER +
			" FROM "
				+ TABLE_EXTERNAL_INSTITUTE + " " + TABLE_ALIAS_EXTERNAL_INSTITUTE +
			" WHERE "
				+ COLUMN_EXTERNAL_INSTITUTE_ID + " = ?";

	private static final String QUERY_PHONES_FOR_DATA_PROVIDER =
			"SELECT "
				+ COLUMN_DP_PHONE_PHONE_NUM + ", "
				+ COLUMN_DP_PHONE_IS_FAX +
			" FROM "
				+ TABLE_DP_PHONE +
			" WHERE "
				+ COLUMN_DP_PHONE_DP_ID + " = ?" +
			" ORDER BY "
				+ COLUMN_DP_PHONE_ID + " ASC";
	private static final String QUERY_PHONES_FOR_EXTERNAL_INSTITUTE =
			"SELECT "
				+ COLUMN_EI_PHONE_PHONE_NUM + ", "
				+ COLUMN_EI_PHONE_IS_FAX +
			" FROM "
				+ TABLE_EI_PHONE +
			" WHERE "
				+ COLUMN_EI_PHONE_EI_ID + " = ?" +
			" ORDER BY "
				+ COLUMN_EI_PHONE_ID + " ASC";

	public DataProviderDisplayInfo fromName(JdbcTemplate jdbcTemplate, String name, String type)
	{
		DataProviderDisplayInfo res;

		Integer id = (Integer) jdbcTemplate.query(QUERY_INSTITUTE_BY_NAME,
				new Object[]{name},
				new ResultSetExtractor()
				{
					public Object extractData(ResultSet rs) throws SQLException
					{
						if (rs.next())
						{
							return new Integer(rs.getInt(1));
						}
						return new Integer(-1);
					}
				});

		if (id != -1)
		{
			res = fromId(jdbcTemplate, id, type);
		}
		else
		{
			res = new DataProviderDisplayInfo();
		}

		return res;
	}
	public Address getAddress(JdbcTemplate jdbcTemplate, long dataProviderId)
	{
		Address address = (Address) jdbcTemplate.query(QUERY_FULL_ADDRESS, new Object[]{dataProviderId}, new ResultSetExtractor()
		{
			public Object extractData(ResultSet resultSet) throws SQLException, DataAccessException
			{
				if (resultSet.next())
				{
					long id = resultSet.getLong("addressId");
					String addr = resultSet.getString("addressAddress");
					long zipCodeId = resultSet.getLong("zipcode_settlement_id");
					String zipcode = resultSet.getString("zipcode");
					long settlementId = resultSet.getLong("settlementId");
					String settlement = resultSet.getString("settlementName");
					Double latitude = resultSet.getDouble("addressLatitude");
					if (resultSet.wasNull())
					{
						latitude = null;
					}
					Double longitude = resultSet.getDouble("addressLongitude");
					if (resultSet.wasNull())
					{
						longitude = null;
					}

					Address address = new Address();
					address.setId(id);
					address.setAddress(addr);
					address.setLatitude(latitude);
					address.setLongitude(longitude);

					ZipCode zipCode = new ZipCode();
					zipCode.setId(zipCodeId);
					zipCode.setZipCode(zipcode);

					Settlement settlementObject = new Settlement();
					settlementObject.setId(settlementId);
					settlementObject.setName(settlement);
					settlementObject.setLongitude(resultSet.getDouble("settlementLongitude"));
					settlementObject.setLatitude(resultSet.getDouble("settlementLatitude"));

					SubRegion subRegion = new SubRegion();
					subRegion.setId(resultSet.getLong("subRegionId"));
					subRegion.setName(resultSet.getString("subRegionName"));
					subRegion.setLongitude(resultSet.getDouble("subRegionLongitude"));
					subRegion.setLatitude(resultSet.getDouble("subRegionLatitude"));

					County county = new County();
					county.setId(resultSet.getLong("countyId"));
					county.setName(resultSet.getString("countyName"));
					county.setLongitude(resultSet.getDouble("countyLongitude"));
					county.setLatitude(resultSet.getDouble("countyLatitude"));

					Region region = new Region();
					region.setId(resultSet.getLong("regionId"));
					region.setName(resultSet.getString("regionName"));
					region.setLongitude(resultSet.getDouble("regionLongitude"));
					region.setLatitude(resultSet.getDouble("regionLatitude"));

					county.setRegion(region);

					subRegion.setCounty(county);

					settlementObject.setSubRegion(subRegion);

					zipCode.setSettlement(settlementObject);

					address.setZipCode(zipCode);

					return address;
				}
				else
				{
					return null;
				}
			}
		});
		return address;
	}

	public DataProviderDisplayInfo fromId(JdbcTemplate jdbcTemplate, int id)
	{
		return fromId(jdbcTemplate, id, TYPE_NAME_ACTIVE_DATAPROVIDER);
	}

	public DataProviderDisplayInfo fromId(JdbcTemplate jdbcTemplate, long id, String type)
	{
		final DataProviderDisplayInfo result = new DataProviderDisplayInfo();

		final List<String> instPhones = new ArrayList<String>();
		result.setPhoneNumber(instPhones);
		final List<String> instFaxes = new ArrayList<String>();
		result.setFaxNumber(instFaxes);

		if (TYPE_NAME_ACTIVE_DATAPROVIDER.equals(type) || TYPE_NAME_REGISTERED_DATAPROVIDER.equals(type))
		{
			getDataProviderById(jdbcTemplate, id, result);
			Address newAddress = getAddress(jdbcTemplate, id);
			if (newAddress != null)
			{
				String zipCode = newAddress.getZipCode().getZipCode();
				String settlement = newAddress.getZipCode().getSettlement().getName();
				String address = newAddress.getAddress();
				StringBuilder newAddressString = new StringBuilder();
				if (zipCode == null || settlement == null || address == null)
				{
					result.setNewAddress("");
				}
				else
				{
					newAddressString.append(zipCode).append(" ");
					newAddressString.append(settlement).append(", ");
					newAddressString.append(address);
					result.setNewAddress(newAddressString.toString());
				}
			}
		}
		else if (TYPE_NAME_EXTERNAL_INSTITUTE.equals(type))
		{
			getExternalInstituteById(jdbcTemplate, id, result);
		}

		return result;
	}

	private void getExternalInstituteById(JdbcTemplate jdbcTemplate, long id, final DataProviderDisplayInfo result)
	{
		jdbcTemplate.query(QUERY_EXTERNAL_INSTITUTE_BY_ID, new Object[]{id}, new ResultSetExtractor()
		{
			public Object extractData(ResultSet resultSet) throws SQLException, DataAccessException
			{
				if (resultSet.next())
				{
					result.setInstitutionName(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_NAME));
					// result.setInstitutionShortName(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_NAME));
					result.setInstitutionShortName(null);
					result.setAddress(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_ADDRESS));
					result.setNewAddress(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_ADDRESS));
					result.setPostalAddress(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_MAIL_ADDRESS));
					result.setWebsite(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_WEB));
					result.setEmail(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_EMAIL));
					result.setLeader(resultSet.getString(COLUMN_EXTERNAL_INSTITUTE_LEADER));
					result.setUserType(null);
					result.setOaiId(null);
					result.setDataProviderId(resultSet.getLong(COLUMN_EXTERNAL_INSTITUTE_ID));
				}
				return null;
			}
		});

		jdbcTemplate.query(QUERY_PHONES_FOR_EXTERNAL_INSTITUTE, new Object[]{id}, new ResultSetExtractor()
		{
			public Object extractData(ResultSet rs) throws SQLException
			{
				while (rs.next())
				{
					if (rs.getBoolean(COLUMN_EI_PHONE_IS_FAX))
					{
						result.getFaxNumber().add(rs.getString(COLUMN_EI_PHONE_PHONE_NUM));
					}
					else
					{
						result.getPhoneNumber().add(rs.getString(COLUMN_EI_PHONE_PHONE_NUM));
					}
				}

				return null;
			}
		});
	}

	private void getDataProviderById(JdbcTemplate jdbcTemplate, long id, final DataProviderDisplayInfo result)
	{
		jdbcTemplate.query(
				QUERY_INSTITUTE_BY_ID,
				new Object[]{id},
				new ResultSetExtractor()
				{
					public Object extractData(ResultSet rs) throws SQLException
					{
						if (rs.next())
						{
							result.setInstitutionName(rs.getString(1));
							result.setInstitutionShortName(rs.getString(2));
							result.setAddress(rs.getString(3));
							result.setPostalAddress(rs.getString(4));
							result.setWebsite(rs.getString(5));
							result.setEmail(rs.getString(6));
							result.setLeader(rs.getString(7));
/*
                        if ( rs.getString( 8 ) != null )
                            instPhones.add( rs.getString( 8 ));

                         if ( rs.getString( 9 ) != null )
                             instFaxes.add( rs.getString( 9 ));
*/
							result.setUserType(rs.getString(10));
							result.setOaiId(rs.getString(11));
							result.setDataProviderId(rs.getLong(12));
							result.setState(rs.getString(13));

						}

						return null;
					}
				});

		final List<String> instTypes = new ArrayList<String>();
		jdbcTemplate.query(
				"select it.type " +
						"from institution_type it, dp_inst_type dit " +
						"where dit.inst_type_id = it.id and " +
						"      dit.dp_id=? " +
						"order by it.id asc",
				new Object[]{id},
				new ResultSetExtractor()
				{
					public Object extractData(ResultSet rs) throws SQLException
					{
						while (rs.next())
						{
							instTypes.add(rs.getString(1));
						}

						return null;
					}
				});
		result.setInstitutionType(instTypes);

		jdbcTemplate.query(
				QUERY_PHONES_FOR_DATA_PROVIDER,
				new Object[]{id},
				new ResultSetExtractor()
				{
					public Object extractData(ResultSet rs) throws SQLException
					{
						while (rs.next())
						{
							if (rs.getBoolean(COLUMN_DP_PHONE_IS_FAX))
							{
								result.getFaxNumber().add(rs.getString(COLUMN_DP_PHONE_PHONE_NUM));
							}
							else
							{
								result.getPhoneNumber().add(rs.getString(COLUMN_DP_PHONE_PHONE_NUM));
							}
						}

						return null;
					}
				});
	}
}
