package hu.kozadat.search.persistence.core;

import hu.scriptum.einfoszab.api.EInfoSzabOperationException;
import hu.scriptum.einfoszab.datamodel.*;
import hu.scriptum.einfoszab.rdbms.RdbmsUserOperations;
import hu.scriptum.einfoszab.rdbms.mapper.InstitutionTypeRowMapper;
import hu.scriptum.einfoszab.rdbms.mapper.PhoneRowMapper;
import hu.scriptum.misc.NormalizerUtilities;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.IncorrectResultSizeDataAccessException;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.RowCallbackHandler;
import org.springframework.jdbc.core.RowMapper;

import java.math.BigDecimal;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

public class RdbmsDataProviderOperations {

    private final static Logger LOGGER = LoggerFactory.getLogger(RdbmsDataProviderOperations.class);

    private static final SimpleDateFormat formatter = new SimpleDateFormat("yyyyMMddHHmmss");

    private JdbcTemplate jdbcTemplate;
    private RdbmsUserOperations userOperations;

    private String modificationUrl;
    private boolean needIpFilter;
    private List harvesterIpList;

    private List supportedSchemas;
    private List defaultSchemas;
    private Set defaultSchemaIds;

    private int defaultProviderPriority = 50;

    private static final String XML_HEADER = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    private static final String XML_ROOTSTART = "<repository xmlns=\"http://www.scriptum.hu/oai/repository/config/codexml\">\n";
    private static final String XML_ROOTEND = "</repository>\n";

    private static final SimpleDateFormat YYYYMMDDHHMMSS_FORMATTER = new SimpleDateFormat("yyyyMMddHHmmss");

    private static final String TABLE_DATA_PROVIDERS = "data_providers";

    private static final String COLUMN_DATA_PROVIDER_ID = "id";
    private static final String COLUMN_DATA_PROVIDER_URL = "url";
    private static final String COLUMN_DATA_PROVIDER_OAI_ID = "oai_id";
    private static final String COLUMN_DATA_PROVIDER_USER_TYPE_ID = "user_type_id";
    private static final String COLUMN_DATA_PROVIDER_DATA_TYPE = "data_type";
    private static final String COLUMN_DATA_PROVIDER_HARVEST_INTERVAL = "harvest_interval";
    private static final String COLUMN_DATA_PROVIDER_HARVEST_CHANGESIGN = "harvest_changesign";
    private static final String COLUMN_DATA_PROVIDER_PRIORITY = "priority";
    private static final String COLUMN_DATA_PROVIDER_STATE = "state";
    private static final String COLUMN_DATA_PROVIDER_LAST_MODIFIED = "last_modified";
    private static final String COLUMN_DATA_PROVIDER_CREATION = "creation";
    private static final String COLUMN_DATA_PROVIDER_ENABLED = "enabled";
    private static final String COLUMN_DATA_PROVIDER_NAME = "name";
    private static final String COLUMN_DATA_PROVIDER_SHORT_NAME = "short_name";
    private static final String COLUMN_DATA_PROVIDER_LEADER = "leader";
    private static final String COLUMN_DATA_PROVIDER_ADDRESS = "address";
    private static final String COLUMN_DATA_PROVIDER_MAIL_ADDRESS = "mail_address";
    private static final String COLUMN_DATA_PROVIDER_PHONE_NUM = "phone_num";
    private static final String COLUMN_DATA_PROVIDER_FAX_NUM = "fax_num";
    private static final String COLUMN_DATA_PROVIDER_WEB = "web";
    private static final String COLUMN_DATA_PROVIDER_EMAIL = "email";
    private static final String COLUMN_DATA_PROVIDER_SUPPORTED_SCHEMAS = "supported_schemas";
    private static final String COLUMN_DATA_PROVIDER_PLAN_USE_REPO = "plan_use_repo";
    private static final String COLUMN_DATA_PROVIDER_PLAN_USE_EDITOR = "plan_use_editor";
    private static final String COLUMN_DATA_PROVIDER_PLAN_USE_WEB = "plan_use_centralweb";
    private static final String COLUMN_DATA_PROVIDER_PLAN_USE_OAI = "plan_use_centraloai";
    private static final String COLUMN_DATA_PROVIDER_EMAIL_NEWSLETTER = "email_newsletter";
    private static final String COLUMN_DATA_PROVIDER_EMAIL_REPORT_SUCCESS = "email_report_success";
    private static final String COLUMN_DATA_PROVIDER_EMAIL_REPORT_TEST = "email_report_test";
    private static final String COLUMN_DATA_PROVIDER_EMAIL_REPORT_UNSUCC = "email_report_unsucc";
    private static final String COLUMN_DATA_PROVIDER_HABIT = "habit";

    private static final String INSERT_DATAPROVIDER =
            "INSERT INTO "
                    + TABLE_DATA_PROVIDERS
                    + "("
                    + COLUMN_DATA_PROVIDER_URL + ", "
                    + COLUMN_DATA_PROVIDER_OAI_ID + ", "
                    + COLUMN_DATA_PROVIDER_USER_TYPE_ID + ", "
                    + COLUMN_DATA_PROVIDER_DATA_TYPE + ","
                    + COLUMN_DATA_PROVIDER_HARVEST_INTERVAL + ", "
                    + COLUMN_DATA_PROVIDER_HARVEST_CHANGESIGN + ", "
                    + COLUMN_DATA_PROVIDER_PRIORITY + ", "
                    + COLUMN_DATA_PROVIDER_STATE + ", "
                    + COLUMN_DATA_PROVIDER_LAST_MODIFIED + ", "
                    + COLUMN_DATA_PROVIDER_CREATION + ", "
                    + COLUMN_DATA_PROVIDER_ENABLED + ", "
                    + COLUMN_DATA_PROVIDER_NAME + ", "
                    + COLUMN_DATA_PROVIDER_SHORT_NAME + ", "
                    + COLUMN_DATA_PROVIDER_LEADER + ", "
                    + COLUMN_DATA_PROVIDER_ADDRESS + ", "
                    + COLUMN_DATA_PROVIDER_MAIL_ADDRESS + ", "
                    + COLUMN_DATA_PROVIDER_PHONE_NUM + ", "
                    + COLUMN_DATA_PROVIDER_FAX_NUM + ", "
                    + COLUMN_DATA_PROVIDER_WEB + ", "
                    + COLUMN_DATA_PROVIDER_EMAIL + ", "
                    + COLUMN_DATA_PROVIDER_SUPPORTED_SCHEMAS + ", "
                    + COLUMN_DATA_PROVIDER_PLAN_USE_REPO + ", "
                    + COLUMN_DATA_PROVIDER_PLAN_USE_EDITOR + ", "
                    + COLUMN_DATA_PROVIDER_PLAN_USE_WEB + ", "
                    + COLUMN_DATA_PROVIDER_PLAN_USE_OAI + ", "
                    + COLUMN_DATA_PROVIDER_EMAIL_NEWSLETTER + ", "
                    + COLUMN_DATA_PROVIDER_HABIT
                    + ") " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

    public static final String TABLE_ZIPCODE = "zipcode_settlement";
    public static final String COLUMN_ZIPCODE_ID = "id";
    public static final String COLUMN_ZIPCODE_ID_ALIAS = "zipcode_id";
    public static final String COLUMN_ZIPCODE_ZIPCODE = "zipcode";
    public static final String COLUMN_ZIPCODE_SETTLEMENT_ID = "settlement_id";

    public static final String TABLE_REGION = "region";
    public static final String COLUMN_REGION_ID = "id";
    public static final String COLUMN_REGION_NAME = "name";

    public static final String TABLE_COUNTY = "county";
    public static final String COLUMN_COUNTY_ID = "id";
    public static final String COLUMN_COUNTY_NAME = "name";
    public static final String COLUMN_COUNTY_REGION_ID = "region_id";

    public static final String TABLE_SUBREGION = "subregion";
    public static final String COLUMN_SUBREGION_ID = "id";
    public static final String COLUMN_SUBREGION_NAME = "name";
    public static final String COLUMN_SUBREGION_ = "county_id";

    public static final String TABLE_SETTLEMENT = "settlement";
    public static final String COLUMN_SETTLEMENT_ID = "id";
    public static final String COLUMN_SETTLEMENT_ID_ALIAS = "settlement_id";
    public static final String COLUMN_SETTLEMENT_NAME = "name";
    public static final String COLUMN_SETTLEMENT_SUBREGION_ID = "subregion_id";

    public static final String ORDER_BY_ZIPCODE_TAIL =
            " ORDER BY z." + COLUMN_ZIPCODE_ZIPCODE;

    public static final String QUERY_ZIPCODE_BY_ID =
            "SELECT "
                    + "z." + COLUMN_ZIPCODE_ID + " as " + COLUMN_ZIPCODE_ID_ALIAS + ", "
                    + "z." + COLUMN_ZIPCODE_ZIPCODE + ", "
                    + "s." + COLUMN_SETTLEMENT_ID + " as " + COLUMN_SETTLEMENT_ID_ALIAS + ", "
                    + "s." + COLUMN_SETTLEMENT_NAME
                    + " FROM "
                    + TABLE_ZIPCODE + " z JOIN " + TABLE_SETTLEMENT + " s ON z." + COLUMN_ZIPCODE_SETTLEMENT_ID + " = s." + COLUMN_SETTLEMENT_ID
                    + " WHERE " + "z." + COLUMN_ZIPCODE_ID + " = ?";

    public static final String QUERY_ZIPCODES_WITH_SETTLEMENT =
            "SELECT "
                    + "z." + COLUMN_ZIPCODE_ID + " as " + COLUMN_ZIPCODE_ID_ALIAS + ", "
                    + "z." + COLUMN_ZIPCODE_ZIPCODE + ", "
                    + "s." + COLUMN_SETTLEMENT_ID + " as " + COLUMN_SETTLEMENT_ID_ALIAS + ", "
                    + "s." + COLUMN_SETTLEMENT_NAME
                    + " FROM "
                    + TABLE_ZIPCODE + " z JOIN " + TABLE_SETTLEMENT + " s ON z." + COLUMN_ZIPCODE_SETTLEMENT_ID + " = s." + COLUMN_SETTLEMENT_ID;

    public static final String QUERY_ZIPCODES_WITH_SETTLEMENT_BY_PATTERN =
            QUERY_ZIPCODES_WITH_SETTLEMENT
                    + " WHERE "
                    + "LOWER(z." + COLUMN_ZIPCODE_ZIPCODE + ") LIKE ?"
                    + " OR LOWER(s." + COLUMN_SETTLEMENT_NAME + ") LIKE ?"
                    + ORDER_BY_ZIPCODE_TAIL;

    private static final String QUERY_FULL_ADDRESS =
            "select "
                    + "a.id" + " as addressId, "
                    + "a.zipcode_settlement_id" + ", "
                    + "a.address" + " as addressAddress, "
                    + "a.latitude" + " as addressLatitude, "
                    + "a.longitude" + " as addressLongitude, "
                    + "z.id" + " as zipCodeId, "
                    + "z.zipcode" + ", "
                    + "s.id" + " as " + "settlementId" + ", "
                    + "s.name" + " as " + "settlementName" + ", "
                    + "s.latitude" + " as settlementLatitude, "
                    + "s.longitude" + " as settlementLongitude, "
                    + "sr.id" + " as subRegionId, "
                    + "sr.name" + " as subRegionName, "
                    + "sr.latitude" + " as subRegionLatitude, "
                    + "sr.longitude" + " as subRegionLongitude, "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude"
                    + " from "
                    + "data_providers dp "
                    + "left join " + "dp_address" + " a " + "on " + "dp.address_id = a.id "
                    + "left join " + "zipcode_settlement" + " z " + "on a.zipcode_settlement_id = z.id "
                    + "left join " + "settlement" + " s " + "on z.settlement_id = s.id "
                    + "left join " + "subregion" + " sr " + "on s.subregion_id = sr.id "
                    + "left join " + "county" + " c " + "on sr.county_id = c.id "
                    + "left join " + "region" + " r " + "on c.region_id = r.id"
                    + " where "
                    + "dp.id=?";

    private static final String QUERY_DATAPROVIDERS_BY =
            "select "
                    + "dp." + COLUMN_DATA_PROVIDER_ID + " as dataProviderId, "
                    + "dp." + COLUMN_DATA_PROVIDER_URL + " as dataProviderUrl, "
                    + "dp." + COLUMN_DATA_PROVIDER_OAI_ID + " as dataProviderOaiId, "
                    + "dp." + COLUMN_DATA_PROVIDER_USER_TYPE_ID + " as dataProviderUserType, "
                    + "dp." + COLUMN_DATA_PROVIDER_DATA_TYPE + " as dataProviderDataType,"
                    + "dp." + COLUMN_DATA_PROVIDER_HARVEST_INTERVAL + " as dataProviderHarvestInterval, "
                    + "dp." + COLUMN_DATA_PROVIDER_HARVEST_CHANGESIGN + " as dataProviderHarvestChangeSign, "
                    + "dp." + COLUMN_DATA_PROVIDER_PRIORITY + " as dataProviderPriority, "
                    + "dp." + COLUMN_DATA_PROVIDER_STATE + " as dataProviderState, "
                    + "dp." + COLUMN_DATA_PROVIDER_LAST_MODIFIED + " as dataProviderLastModified, "
                    + "dp." + COLUMN_DATA_PROVIDER_CREATION + " as dataProviderCreation, "
                    + "dp." + COLUMN_DATA_PROVIDER_ENABLED + " as dataProviderEnabled, "
                    + "dp." + COLUMN_DATA_PROVIDER_NAME + " as dataProviderName, "
                    + "dp." + COLUMN_DATA_PROVIDER_SHORT_NAME + " as dataProviderShortName, "
                    + "dp." + COLUMN_DATA_PROVIDER_LEADER + " as dataProviderLeader, "
                    + "dp." + COLUMN_DATA_PROVIDER_ADDRESS + " as dataProviderAddress, "
                    + "dp." + COLUMN_DATA_PROVIDER_MAIL_ADDRESS + " as dataProviderMailAddress, "
                    + "dp." + COLUMN_DATA_PROVIDER_PHONE_NUM + "  as dataProviderPhoneNum, "
                    + "dp." + COLUMN_DATA_PROVIDER_FAX_NUM + "  as dataProviderFaxNum, "
                    + "dp." + COLUMN_DATA_PROVIDER_WEB + "  as dataProviderWeb, "
                    + "dp." + COLUMN_DATA_PROVIDER_EMAIL + " as dataProviderEmail, "
                    + "dp." + COLUMN_DATA_PROVIDER_SUPPORTED_SCHEMAS + "  as dataProviderSupportedSchemas, "
                    + "dp." + COLUMN_DATA_PROVIDER_PLAN_USE_REPO + " as dataProviderPlanUseRepo, "
                    + "dp." + COLUMN_DATA_PROVIDER_PLAN_USE_EDITOR + " as dataProviderPlanUseEditor, "
                    + "dp." + COLUMN_DATA_PROVIDER_PLAN_USE_WEB + " as dataProviderPlanUseWeb, "
                    + "dp." + COLUMN_DATA_PROVIDER_PLAN_USE_OAI + " as dataProviderPlanUseOai, "
                    + "dp." + COLUMN_DATA_PROVIDER_EMAIL_NEWSLETTER + " as dataProviderEmailNewsletter, "
                    + "dp." + COLUMN_DATA_PROVIDER_HABIT + " as dataProviderHabit, "
                    + "a.id" + " as addressId, "
                    + "a.zipcode_settlement_id" + ", "
                    + "a.address" + " as addressAddress, "
                    + "a.latitude" + " as addressLatitude, "
                    + "a.longitude" + " as addressLongitude, "
                    + "z.id" + " as zipCodeId, "
                    + "z.zipcode" + ", "
                    + "s.id" + " as " + "settlementId" + ", "
                    + "s.name" + " as " + "settlementName" + ", "
                    + "s.latitude" + " as settlementLatitude, "
                    + "s.longitude" + " as settlementLongitude, "
                    + "sr.id" + " as subRegionId, "
                    + "sr.name" + " as subRegionName, "
                    + "sr.latitude" + " as subRegionLatitude, "
                    + "sr.longitude" + " as subRegionLongitude, "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude" +
                    " from "
                    + "data_providers dp "
                    + "inner join " + "dp_address" + " a " + "on " + "dp.address_id = a.id "
                    + "inner join " + "zipcode_settlement" + " z " + "on a.zipcode_settlement_id = z.id "
                    + "inner join " + "settlement" + " s " + "on z.settlement_id = s.id "
                    + "inner join " + "subregion" + " sr " + "on s.subregion_id = sr.id "
                    + "inner join " + "county" + " c " + "on sr.county_id = c.id "
                    + "inner join " + "region" + " r " + "on c.region_id = r.id";


    private static final String QUERY_DATAPROVIDERS_BY_SETTLEMENT =
            QUERY_DATAPROVIDERS_BY
                    + " where "
                    + "s.id=?" + " and " + "dp." + COLUMN_DATA_PROVIDER_STATE + " in ('A', 'R')";

    private static final String QUERY_DATAPROVIDERS_BY_SUBREGION =
            QUERY_DATAPROVIDERS_BY
                    + " where "
                    + "sr.id=?" + " and " + "dp." + COLUMN_DATA_PROVIDER_STATE + " in ('A', 'R')";

    private static final String QUERY_DATAPROVIDERS_BY_COUNTY =
            QUERY_DATAPROVIDERS_BY
                    + " where "
                    + "c.id=?" + " and " + "dp." + COLUMN_DATA_PROVIDER_STATE + " in ('A', 'R')";

    private static final String QUERY_SUBREGIONS_BY_ID =
            "select "
                    + "sr.id" + " as subRegionId, "
                    + "sr.name" + " as subRegionName, "
                    + "sr.latitude" + " as subRegionLatitude, "
                    + "sr.longitude" + " as subRegionLongitude, "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude" +
                    " from "
                    + "subregion" + " sr "
                    + "inner join " + "county" + " c " + "on sr.county_id = c.id "
                    + "inner join " + "region" + " r " + "on c.region_id = r.id" +
                    " where "
                    + "sr.id=?";

    private static final String QUERY_SUBREGIONS_BY_COUNTY =
            "select "
                    + "sr.id" + " as subRegionId, "
                    + "sr.name" + " as subRegionName, "
                    + "sr.latitude" + " as subRegionLatitude, "
                    + "sr.longitude" + " as subRegionLongitude, "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude" +
                    " from "
                    + "subregion" + " sr "
                    + "inner join " + "county" + " c " + "on sr.county_id = c.id "
                    + "inner join " + "region" + " r " + "on c.region_id = r.id" +
                    " where "
                    + "c.id=?";

    private static final String QUERY_SUBREGIONS_BY_REGION =
            "select "
                    + "sr.id" + " as subRegionId, "
                    + "sr.name" + " as subRegionName, "
                    + "sr.latitude" + " as subRegionLatitude, "
                    + "sr.longitude" + " as subRegionLongitude, "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude" +
                    " from "
                    + "subregion" + " sr "
                    + "inner join " + "county" + " c " + "on sr.county_id = c.id "
                    + "inner join " + "region" + " r " + "on c.region_id = r.id" +
                    " where "
                    + "r.id=?";

    private static final String QUERY_REGIONS =
            "select "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude "
                    + "from "
                    + "region" + " r";

    private static final String QUERY_REGION =
            "select "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude "
                    + "from "
                    + "region" + " r "
                    + "where r.id=?";

    private static final String QUERY_COUNTY =
            "select "
                    + "c.id" + " as countyId, "
                    + "c.name" + " as countyName, "
                    + "c.latitude" + " as countyLatitude, "
                    + "c.longitude" + " as countyLongitude, "
                    + "r.id" + " as regionId, "
                    + "r.name" + " as regionName, "
                    + "r.latitude" + " as regionLatitude, "
                    + "r.longitude" + " as regionLongitude" +
                    " from "
                    + "county" + " c "
                    + "inner join " + "region" + " r " + "on c.region_id = r.id" +
                    " where "
                    + "c.id=?";

    public List<DataProvider> getDataProvidersInSettlement(long settlementId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getDataProvidersInSettlement() CALLED, params: settlementId=" + settlementId);
        }
        List<DataProvider> dataProviders = new LinkedList<DataProvider>();

        Collection queryResult = jdbcTemplate.query(QUERY_DATAPROVIDERS_BY_SETTLEMENT, new Object[]{settlementId}, new DataProviderMapper());
        for (Object o : queryResult) {
            if (o instanceof DataProvider) {
                dataProviders.add((DataProvider) o);
            }
        }

        LOGGER.debug("getDataProvidersInSettlement() FINISHED");

        return dataProviders;
    }

    public List<DataProvider> getDataProvidersInSubRegion(long subRegionId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getDataProvidersInSubRegion() CALLED, params: subRegionId=" + subRegionId);
        }
        List<DataProvider> dataProviders = new LinkedList<DataProvider>();

        Collection queryResult = jdbcTemplate.query(QUERY_DATAPROVIDERS_BY_SUBREGION, new Object[]{subRegionId}, new DataProviderMapper());
        for (Object o : queryResult) {
            if (o instanceof DataProvider) {
                dataProviders.add((DataProvider) o);
            }
        }

        LOGGER.debug("getDataProvidersInSubRegion() FINISHED");

        return dataProviders;
    }

    public Map<Long, Long> getDataProviderCountInSubRegionsByCounty(long countyId) {
        long start = System.currentTimeMillis();
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getDataProviderCountInSubRegionsByCounty() CALLED, params: countyId=" + countyId);
        }
        Map<Long, Long> resultMap = null;

        String statement =
                "select "
                        + "sr.id" + ", "
                        + "count(dp.oai_id)"
                        + " from "
                        + "data_providers dp "
                        + "left join " + "dp_address" + " a " + "on " + "dp.address_id = a.id "
                        + "left join " + "zipcode_settlement" + " z " + "on a.zipcode_settlement_id = z.id "
                        + "left join " + "settlement" + " s " + "on z.settlement_id = s.id "
                        + "left join " + "subregion" + " sr " + "on s.subregion_id = sr.id "
                        + "left join " + "county" + " c " + "on sr.county_id = c.id "
                        + "left join " + "region" + " r " + "on c.region_id = r.id"
                        + " where " + "dp." + COLUMN_DATA_PROVIDER_STATE + " in ('A', 'R')"
                        + " group by " + "sr.id" + ", " + "c.id"
                        + " having " + "c.id" + "=?";

        LOGGER.debug("statement=" + statement);

        if (countyId > 0) {
            Collection resultCollection = jdbcTemplate.query(statement, new Object[]{countyId}, new CountMapper());
            if (resultCollection != null) {
                resultMap = new HashMap<Long, Long>(resultCollection.size());
                for (Object o : resultCollection) {
                    if (o instanceof CountObject) {
                        CountObject countObject = (CountObject) o;
                        resultMap.put(countObject.getId(), countObject.getCount());
                    }
                }

            }
        }

        LOGGER.debug("getDataProviderCountInSubRegionsByCounty() FINISHED in " + (System.currentTimeMillis() - start) + " ms");

        return resultMap;
    }

    public Map<Long, Long> getDataProviderCountInSubRegionsByRegion(long regionId) {
        long start = System.currentTimeMillis();
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getDataProviderCountInSubRegionsByRegion() CALLED, params: regionId=" + regionId);
        }
        Map<Long, Long> resultMap = null;

        String statement =
                "select "
                        + "sr.id" + ", "
                        + "count(dp.oai_id)"
                        + " from "
                        + "data_providers dp "
                        + "left join " + "dp_address" + " a " + "on " + "dp.address_id = a.id "
                        + "left join " + "zipcode_settlement" + " z " + "on a.zipcode_settlement_id = z.id "
                        + "left join " + "settlement" + " s " + "on z.settlement_id = s.id "
                        + "left join " + "subregion" + " sr " + "on s.subregion_id = sr.id "
                        + "left join " + "county" + " c " + "on sr.county_id = c.id "
                        + "left join " + "region" + " r " + "on c.region_id = r.id"
                        + " where " + "dp." + COLUMN_DATA_PROVIDER_STATE + " in ('A', 'R')"
                        + " group by " + "sr.id" + ", " + "r.id"
                        + " having " + "r.id" + "=?";

        LOGGER.debug("statement=" + statement);

        if (regionId > 0) {
            Collection resultCollection = jdbcTemplate.query(statement, new Object[]{regionId}, new CountMapper());
            if (resultCollection != null) {
                resultMap = new HashMap<Long, Long>(resultCollection.size());
                for (Object o : resultCollection) {
                    if (o instanceof CountObject) {
                        CountObject countObject = (CountObject) o;
                        resultMap.put(countObject.getId(), countObject.getCount());
                    }
                }

            }
        }

        LOGGER.debug("getDataProviderCountInSubRegionsByRegion() FINISHED in " + (System.currentTimeMillis() - start) + " ms");

        return resultMap;
    }

    public List<DataProvider> getDataProvidersInCounty(long countyId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getDataProvidersInCounty() CALLED, params: countyId=" + countyId);
        }
        List<DataProvider> dataProviders = new LinkedList<DataProvider>();

        Collection queryResult = jdbcTemplate.query(QUERY_DATAPROVIDERS_BY_COUNTY, new Object[]{countyId}, new DataProviderMapper());
        for (Object o : queryResult) {
            if (o instanceof DataProvider) {
                dataProviders.add((DataProvider) o);
            }
        }

        LOGGER.debug("getDataProvidersInCounty() FINISHED");

        return dataProviders;
    }

    public SubRegion getSubRegion(long subRegionId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getSubRegion() CALLED, params: subRegionId=" + subRegionId);
        }
        SubRegion result = null;
        try {
            Object o = jdbcTemplate.queryForObject(QUERY_SUBREGIONS_BY_ID, new Object[]{subRegionId}, new SubRegionMapper());
            if (o instanceof SubRegion) {
                result = (SubRegion) o;
            }
        } catch (IncorrectResultSizeDataAccessException e) {
            LOGGER.warn("subRegionId=" + subRegionId + ", " + e.getMessage());
        }

        LOGGER.debug("getSubRegion() FINISHED");

        return result;
    }

    public List<SubRegion> getSubRegionsInCounty(long countyId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getSubRegionsInCounty() CALLED, params: countyId=" + countyId);
        }
        List<SubRegion> subRegions = new LinkedList<SubRegion>();

        Collection queryResult = jdbcTemplate.query(QUERY_SUBREGIONS_BY_COUNTY, new Object[]{countyId}, new SubRegionMapper());
        for (Object o : queryResult) {
            if (o instanceof SubRegion) {
                subRegions.add((SubRegion) o);
            }
        }

        LOGGER.debug("getSubRegionsInCounty() FINISHED");
        return subRegions;
    }

    public List<SubRegion> getSubRegionsInRegion(long regionId) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getSubRegionsInRegion() CALLED, params: regionId=" + regionId);
        }

        List<SubRegion> subRegions = new LinkedList<SubRegion>();

        Collection queryResult = jdbcTemplate.query(QUERY_SUBREGIONS_BY_REGION, new Object[]{regionId}, new SubRegionMapper());
        for (Object o : queryResult) {
            if (o instanceof SubRegion) {
                subRegions.add((SubRegion) o);
            }
        }

        LOGGER.debug("getSubRegionsInRegion() FINISHED");
        return subRegions;
    }

    public Region getRegion(Long regionId) {
        Region region = null;
        try {
            region = jdbcTemplate.queryForObject(QUERY_REGION, new Object[]{regionId}, new RegionMapper());
        } catch (IncorrectResultSizeDataAccessException e) {
            LOGGER.warn("regionId=" + regionId + ", " + e.getMessage());
        }
        return region;
    }

    public County getCounty(Long countyId) {
        County county = null;
        try {
            county = jdbcTemplate.queryForObject(QUERY_COUNTY, new Object[]{countyId}, new CountyMapper());
        } catch (IncorrectResultSizeDataAccessException e) {
            LOGGER.warn("countyId=" + countyId + ", " + e.getMessage());
        }
        return county;
    }

    public List<Region> getRegions() {
        List<Region> regions = jdbcTemplate.query(QUERY_REGIONS, new RegionMapper());
        return regions;
    }

    private static final class ZipCodeMapper implements RowMapper {

        public Object mapRow(ResultSet resultSet, int i) throws SQLException {
            ZipCode zipCode = new ZipCode();

            zipCode.setId(resultSet.getLong(COLUMN_ZIPCODE_ID_ALIAS));
            zipCode.setZipCode(resultSet.getString(COLUMN_ZIPCODE_ZIPCODE));

            Settlement settlement = new Settlement();
            settlement.setId(resultSet.getLong(COLUMN_SETTLEMENT_ID_ALIAS));
            settlement.setName(resultSet.getString(COLUMN_SETTLEMENT_NAME));

            zipCode.setSettlement(settlement);

            return zipCode;
        }
    }

    private static final class CountObject {
        private long id;
        private long count;

        public CountObject() {
        }

        public CountObject(long id, long count) {
            this.id = id;
            this.count = count;
        }

        public long getId() {
            return id;
        }

        public void setId(long id) {
            this.id = id;
        }

        public long getCount() {
            return count;
        }

        public void setCount(long count) {
            this.count = count;
        }
    }

    private static final class CountMapper implements RowMapper {

        public Object mapRow(ResultSet resultSet, int i) throws SQLException {
            // 1 - id
            // 2 - count
            CountObject countObject = new CountObject(resultSet.getLong(1), resultSet.getLong(2));
            return countObject;
        }
    }

    private static final class DataProviderMapper implements RowMapper {

        public Object mapRow(ResultSet resultSet, int i) throws SQLException {
            DataProvider dataProvider = new DataProvider();

            dataProvider.setId(resultSet.getLong("dataProviderId"));
            dataProvider.setUrl(resultSet.getString("dataProviderUrl"));
            dataProvider.setOaiIdentifier(resultSet.getString("dataProviderOaiId"));
            dataProvider.setPriority(resultSet.getInt("dataProviderPriority"));
            dataProvider.setState(resultSet.getString("dataProviderState"));
            dataProvider.setLastModified(parseDate(resultSet.getString("dataProviderLastModified")));
            dataProvider.setDisabled("N".equals(resultSet.getString("dataProviderEnabled")));
            dataProvider.setHarvestChangeSignSupported("Y".equals(resultSet.getString("dataProviderHarvestChangeSign")));
            dataProvider.setPlanUseCentralWeb("Y".equals(resultSet.getString("dataProviderPlanUseWeb")));
            dataProvider.setPlanUseCentralOAI("Y".equals(resultSet.getString("dataProviderPlanUseOai")));
            dataProvider.setPlanUseDefaultEditor("Y".equals(resultSet.getString("dataProviderPlanUseEditor")));
            dataProvider.setPlanUseDefaultRepository("Y".equals(resultSet.getString("dataProviderPlanUseRepo")));
            dataProvider.setEmailNewsletter("Y".equals(resultSet.getString("dataProviderEmailNewsletter")));
            dataProvider.setHabit(new DataProviderHabit(resultSet.getInt("dataProviderHabit")));
            dataProvider.setHarvestInterval(resultSet.getInt("dataProviderHarvestInterval"));

            final Institution institution = new Institution();
            institution.setName(resultSet.getString("dataProviderName"));
            institution.setShortName(resultSet.getString("dataProviderShortName"));
            institution.setAddress(resultSet.getString("dataProviderAddress"));
            institution.setMailAddress(resultSet.getString("dataProviderMailAddress"));
            institution.setFaxNumber(resultSet.getString("dataProviderFaxNum"));
            institution.setPhoneNumber(resultSet.getString("dataProviderPhoneNum"));
            institution.setEmail(resultSet.getString("dataProviderEmail"));
            institution.setWebAddress(resultSet.getString("dataProviderWeb"));
            institution.setLeaderName(resultSet.getString("dataProviderLeader"));

            Address address = new Address();

            address.setId(resultSet.getLong("addressId"));
            address.setAddress(resultSet.getString("addressAddress"));
            address.setLongitude(resultSet.getDouble("addressLongitude"));
            address.setLatitude(resultSet.getDouble("addressLatitude"));

            ZipCode zipCode = new ZipCode();
            zipCode.setId(resultSet.getLong("zipCodeId"));
            zipCode.setZipCode(resultSet.getString("zipcode"));

            Settlement settlement = new Settlement();
            settlement.setId(resultSet.getLong("settlementId"));
            settlement.setName(resultSet.getString("settlementName"));
            settlement.setLongitude(resultSet.getDouble("settlementLongitude"));
            settlement.setLatitude(resultSet.getDouble("settlementLatitude"));

            SubRegion subRegion = new SubRegion();
            subRegion.setId(resultSet.getLong("subRegionId"));
            subRegion.setName(resultSet.getString("subRegionName"));
            subRegion.setLongitude(resultSet.getDouble("subRegionLongitude"));
            subRegion.setLatitude(resultSet.getDouble("subRegionLatitude"));

            County county = new County();
            county.setId(resultSet.getLong("countyId"));
            county.setName(resultSet.getString("countyName"));
            county.setLongitude(resultSet.getDouble("countyLongitude"));
            county.setLatitude(resultSet.getDouble("countyLatitude"));

            Region region = new Region();
            region.setId(resultSet.getLong("regionId"));
            region.setName(resultSet.getString("regionName"));
            region.setLongitude(resultSet.getDouble("regionLongitude"));
            region.setLatitude(resultSet.getDouble("regionLatitude"));

            county.setRegion(region);

            subRegion.setCounty(county);

            settlement.setSubRegion(subRegion);

            zipCode.setSettlement(settlement);

            address.setZipCode(zipCode);

            institution.setNewAddress(address);

            dataProvider.setInstitution(institution);

            return dataProvider;
        }
    }

    private static final class SubRegionMapper implements RowMapper {

        public Object mapRow(ResultSet resultSet, int i) throws SQLException {
            SubRegion subRegion = new SubRegion();
            subRegion.setId(resultSet.getLong("subRegionId"));
            subRegion.setName(resultSet.getString("subRegionName"));
            subRegion.setLongitude(resultSet.getDouble("subRegionLongitude"));
            subRegion.setLatitude(resultSet.getDouble("subRegionLatitude"));

            County county = new County();
            county.setId(resultSet.getLong("countyId"));
            county.setName(resultSet.getString("countyName"));
            county.setLongitude(resultSet.getDouble("countyLongitude"));
            county.setLatitude(resultSet.getDouble("countyLatitude"));

            Region region = new Region();
            region.setId(resultSet.getLong("regionId"));
            region.setName(resultSet.getString("regionName"));
            region.setLongitude(resultSet.getDouble("regionLongitude"));
            region.setLatitude(resultSet.getDouble("regionLatitude"));

            county.setRegion(region);

            subRegion.setCounty(county);

            return subRegion;
        }
    }

    private static final class RegionMapper implements RowMapper<Region> {

        public Region mapRow(ResultSet resultSet, int i) throws SQLException {
            Region region = new Region();
            region.setId(resultSet.getLong("regionId"));
            region.setName(resultSet.getString("regionName"));
            region.setLongitude(resultSet.getDouble("regionLongitude"));
            region.setLatitude(resultSet.getDouble("regionLatitude"));

            return region;
        }
    }

    private static final class CountyMapper implements RowMapper<County> {

        public County mapRow(ResultSet resultSet, int i) throws SQLException {
            County county = new County();
            county.setId(resultSet.getLong("countyId"));
            county.setName(resultSet.getString("countyName"));
            county.setLongitude(resultSet.getDouble("countyLongitude"));
            county.setLatitude(resultSet.getDouble("countyLatitude"));

            Region region = new Region();
            region.setId(resultSet.getLong("regionId"));
            region.setName(resultSet.getString("regionName"));
            region.setLongitude(resultSet.getDouble("regionLongitude"));
            region.setLatitude(resultSet.getDouble("regionLatitude"));

            county.setRegion(region);

            return county;
        }
    }

    public void setDefaultProviderPriority(int defaultProviderPriority) {
        this.defaultProviderPriority = defaultProviderPriority;
    }

    private static String formatDate(Date date) {
        synchronized (YYYYMMDDHHMMSS_FORMATTER) {
            return YYYYMMDDHHMMSS_FORMATTER.format(date);
        }
    }

    public static Date parseDate(String date) {
        try {
            synchronized (YYYYMMDDHHMMSS_FORMATTER) {
                return YYYYMMDDHHMMSS_FORMATTER.parse(date);
            }
        } catch (ParseException e) {
            throw new RuntimeException("invalid date");
        }
    }

    private String getSchemaRepresentation(List schemaList) {
        StringBuffer buf = new StringBuffer();

        for (Iterator iter = schemaList.iterator(); iter.hasNext(); ) {
            Schema schema = (Schema) iter.next();
            buf.append(schema.getId());
            if (iter.hasNext()) {
                buf.append('|');
            }
        }

        return buf.toString();
    }

    private List getSchemaListFromRepresentation(String representation) {
        List returnValue = new LinkedList();

        if (representation != null) {
            String[] values = representation.split("\\|");

            for (Iterator iter = getSupportedSchemas().iterator(); iter.hasNext(); ) {
                Schema schema = (Schema) iter.next();
                String sId = Long.toString(schema.getId());
                for (int i = 0; i < values.length; i++) {
                    if (values[i].equals(sId)) {
                        returnValue.add(schema);
                        break;
                    }
                }
            }
        }

        return returnValue;
    }

    private Long getUserTypeId(String userType) {
        Long userTypeId = new Long(jdbcTemplate.queryForLong("select id from user_types where user_type=?", new Object[]{userType}));
        return userTypeId;
    }

    public void addDataProvider2(DataProvider dataProvider) throws EInfoSzabOperationException {
        dataProvider.setState(DataProvider.STATE_PRE_REGISTERED);
        dataProvider.setLastModified(new Date());
        dataProvider.setCreation(new Date());

        LOGGER.debug("dataProvider.getUrl(): " + dataProvider.getUrl());
        LOGGER.debug("dataProvider.getOaiIdentifier(): " + dataProvider.getOaiIdentifier());
        LOGGER.debug("user type: " + dataProvider.getUserType() + " (id: " + getUserTypeId(dataProvider.getUserType()) + ")");
        LOGGER.debug("dataProvider.getHarvestInterval(): " + dataProvider.getHarvestInterval());
        LOGGER.debug("dataProvider.isHarvestChangeSignSupported(): " + dataProvider.isHarvestChangeSignSupported());
        LOGGER.debug("dataProvider.getPriority(): " + dataProvider.getPriority());
        LOGGER.debug("dataProvider.getState(): " + dataProvider.getState());
        LOGGER.debug("dataProvider.getLastModified(): " + dataProvider.getLastModified());
        LOGGER.debug("dataProvider.getCreation(): " + dataProvider.getCreation());
        LOGGER.debug("dataProvider.isDisabled(): " + dataProvider.isDisabled());


        LOGGER.debug("dataProvider.getInstitution().getName(): " + dataProvider.getInstitution().getName());
        LOGGER.debug("dataProvider.getInstitution().getShortName(): " + dataProvider.getInstitution().getShortName());
        LOGGER.debug("dataProvider.getInstitution().getLeaderName(): " + dataProvider.getInstitution().getLeaderName());
        LOGGER.debug("dataProvider.getInstitution().getAddress(): " + dataProvider.getInstitution().getAddress());
        LOGGER.debug("dataProvider.getInstitution().getMailAddress(): " + dataProvider.getInstitution().getMailAddress());
        LOGGER.debug("dataProvider.getInstitution().getPhoneNumber(): " + dataProvider.getInstitution().getPhoneNumber());
        LOGGER.debug("dataProvider.getInstitution().getFaxNumber(): " + dataProvider.getInstitution().getFaxNumber());
        LOGGER.debug("dataProvider.getInstitution().getWebAddress(): " + dataProvider.getInstitution().getWebAddress());
        LOGGER.debug("dataProvider.getInstitution().getEmail(): " + dataProvider.getInstitution().getEmail());
        LOGGER.debug("dataProvider.isPlanUseDefaultRepository(): " + dataProvider.isPlanUseDefaultRepository());
        LOGGER.debug("dataProvider.isPlanUseDefaultEditor(): " + dataProvider.isPlanUseDefaultEditor());
        LOGGER.debug("dataProvider.isPlanUseCentralWeb(): " + dataProvider.isPlanUseCentralWeb());
        LOGGER.debug("dataProvider.isPlanUseCentralOAI(): " + dataProvider.isPlanUseCentralOAI());
        LOGGER.debug("dataProvider.isEmailNewsletter(): " + dataProvider.isEmailNewsletter());
        LOGGER.debug("dataProvider.getHabit(): " + dataProvider.getHabit());

        jdbcTemplate.update("insert into data_providers(url, oai_id, user_type_id, data_type, " +
                "harvest_interval, harvest_changesign, priority, state, last_modified, creation, enabled, name, short_name, " +
                "leader, address, mail_address, web, email, " +
                "plan_use_repo, plan_use_editor, plan_use_centralweb, plan_use_centraloai, email_newsletter, habit, " +
                "email_report_success, email_report_unsucc, email_report_test, supported_schemas) " +
                "values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
                new Object[]{
                        dataProvider.getUrl(),
                        dataProvider.getOaiIdentifier(),
                        getUserTypeId(dataProvider.getUserType()),
                        "S",
                        new Integer(dataProvider.getHarvestInterval()),
                        dataProvider.isHarvestChangeSignSupported() ? "Y" : "N",
                        new Integer(dataProvider.getPriority() == 0 ? defaultProviderPriority : dataProvider.getPriority()),
                        dataProvider.getState(),
                        formatDate(dataProvider.getLastModified()),
                        formatDate(dataProvider.getCreation()),
                        dataProvider.isDisabled() ? "N" : "Y",
                        dataProvider.getInstitution().getName(),
                        dataProvider.getInstitution().getShortName(),
                        dataProvider.getInstitution().getLeaderName(),
                        dataProvider.getInstitution().getAddress(),
                        dataProvider.getInstitution().getMailAddress(),
                        dataProvider.getInstitution().getWebAddress(),
                        dataProvider.getInstitution().getEmail(),
                        dataProvider.isPlanUseDefaultRepository() ? "Y" : "N",
                        dataProvider.isPlanUseDefaultEditor() ? "Y" : "N",
                        dataProvider.isPlanUseCentralWeb() ? "Y" : "N",
                        dataProvider.isPlanUseCentralOAI() ? "Y" : "N",
                        dataProvider.isEmailNewsletter() ? "Y" : "N",
                        dataProvider.getHabit().getValue(),
                        dataProvider.getEmailReportSuccess(),
                        dataProvider.getEmailReportUnsucc(),
                        dataProvider.getEmailReportTest(),
                        getSchemaRepresentation(dataProvider.getSupportedSchemas())
                });

        //beállítjuk az újonnan insert-ált data provider-en a kapott id-t
        long dpId = jdbcTemplate.queryForLong("SELECT data_providers.CURRVAL FROM DUAL");
        dataProvider.setId(dpId);

        // save new address
        Address address = dataProvider.getInstitution().getNewAddress();
        LOGGER.debug("save new address: " + address);
        jdbcTemplate.update("insert into dp_address(zipcode_settlement_id, address, longitude, latitude) values (?, ?, ?, ?)", new Object[]{address.getZipCode().getId(), address.getAddress(), address.getLongitude(), address.getLatitude()});
        jdbcTemplate.update("update data_providers set address_id=(select dp_address.currval from dual) where id=?", new Object[]{dataProvider.getId()});

        //intézmény típusok beállítása
        addInstitutionTypes(dataProvider);

        //az intézményi telefonszámok persist-álása
        addDPPhones(dataProvider);

        //a kapcsolattartó persist-álása
        Person responsible = dataProvider.getInstitution().getResponsible();
        if (responsible != null) {

            responsible.setType(Person.TYPE_RESPONSIBLE);
            responsible.setSourceId(dpId);
            responsible.setSourceName(dataProvider.getInstitution().getName());
            responsible.setUserGroup(Person.USERGROUP_DATAPROVIDER);
            userOperations.createPersonInternal(responsible);
        }

        //a kontakt személyek persist-álása
        List contacts = dataProvider.getInstitution().getContacts();
        if (contacts != null) {
            for (Object o : contacts) {
                Person person = (Person) o;
                person.setType(Person.TYPE_CONTACT);
                person.setSourceId(dpId);
                person.setSourceName(dataProvider.getInstitution().getName());
                person.setUserGroup(Person.USERGROUP_DATAPROVIDER);
                userOperations.createPersonInternal(person);
            }
        }


    }

    public void addDPPhones(DataProvider dp) {
        if (dp.getInstitution().getPhones().size() > 0 || dp.getInstitution().getFaxes().size() > 0) {

            List<String> commands = new ArrayList<String>();
            for (String aPhone : dp.getInstitution().getPhones()) {
                if (!"".equals(aPhone)) {
                    StringBuffer sql = new StringBuffer("insert into dp_phone (dp_id, phone_num, is_fax) values (");
                    sql.append(dp.getId() + ", ");
                    sql.append("'" + aPhone + "', ");
                    sql.append("FALSE)");

                    LOGGER.debug("sql added to batch: " + sql.toString());
                    commands.add(sql.toString());
                }
            }

            for (String aFax : dp.getInstitution().getFaxes()) {
                if (!"".equals(aFax)) {
                    StringBuffer sql = new StringBuffer("insert into dp_phone (dp_id, phone_num, is_fax) values (");
                    sql.append(dp.getId() + ", ");
                    sql.append("'" + aFax + "', ");
                    sql.append("TRUE)");

                    LOGGER.debug("sql added to batch: " + sql.toString());
                    commands.add(sql.toString());
                }
            }

            String[] array = new String[commands.size()];
            commands.toArray(array);

            int[] inserted = jdbcTemplate.batchUpdate(array);

            LOGGER.debug(inserted.length + " row(s) inserted into 'user_phone'");
        } else {
            LOGGER.debug("no phone neither fax numbers to persist for this data provider: " + dp);
        }

    }

    public int deleteDPPhones(long dpId) {
        String sql = "delete from dp_phone where dp_id=?";
        return jdbcTemplate.update(sql, new Object[]{dpId});
    }

    public void addInstitutionTypes(DataProvider dp) {
        List<Integer> ids = getInstitutionTypeIds(new ArrayList<String>(dp.getInstitution().getTypes()));
        LOGGER.debug("these institution type IDs will be set for data provider: " + ids);

        for (Integer instTypeId : ids) {
            String sql = "insert into dp_inst_type (dp_id, inst_type_id) values (?, ?)";
            int affectedRows = jdbcTemplate.update(sql, new Object[]{dp.getId(), instTypeId});
            LOGGER.debug("institution type id added: " + instTypeId);
        }
    }

    public int deleteInstitutionTypes(long dpId) {
        String sql = "delete from dp_inst_type where dp_id=?";
        return jdbcTemplate.update(sql, new Object[]{dpId});
    }

    public List<Integer> getInstitutionTypeIds(List<String> types) {
        String sql = "select id from institution_type where type=?";
        List<Integer> ids = new ArrayList<Integer>();
        for (String aType : types) {
            ids.add(jdbcTemplate.queryForInt(sql, new Object[]{aType}));
        }

        return ids;
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#addDataProvider(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public synchronized void addDataProvider(DataProvider dataProvider) throws EInfoSzabOperationException {
        dataProvider.setState(DataProvider.STATE_PRE_REGISTERED);
        dataProvider.setLastModified(new Date());

        Long userTypeId = null;
        if (dataProvider.getUserType() != null) {
            userTypeId = getUserTypeId(dataProvider.getUserType());
        }

        try {
            jdbcTemplate.update(INSERT_DATAPROVIDER,
                    new Object[]{
                            dataProvider.getUrl(),
                            dataProvider.getOaiIdentifier(),
                            userTypeId,
                            "S",
                            new Integer(dataProvider.getHarvestInterval()),
                            dataProvider.isHarvestChangeSignSupported() ? "Y" : "N",
                            new Integer(dataProvider.getPriority() == 0 ? defaultProviderPriority : dataProvider.getPriority()),
                            dataProvider.getState(),
                            formatDate(dataProvider.getLastModified()),
                            formatDate(new Date()),
                            dataProvider.isDisabled() ? "N" : "Y",
                            dataProvider.getInstitution().getName(),
                            dataProvider.getInstitution().getShortName(),
                            dataProvider.getInstitution().getLeaderName(),
                            dataProvider.getInstitution().getAddress(),
                            dataProvider.getInstitution().getMailAddress(),
                            dataProvider.getInstitution().getPhoneNumber(),
                            dataProvider.getInstitution().getFaxNumber(),
                            dataProvider.getInstitution().getWebAddress(),
                            dataProvider.getInstitution().getEmail(),
                            getSchemaRepresentation(dataProvider.getSupportedSchemas()),
                            dataProvider.isPlanUseDefaultRepository() ? "Y" : "N",
                            dataProvider.isPlanUseDefaultEditor() ? "Y" : "N",
                            dataProvider.isPlanUseCentralWeb() ? "Y" : "N",
                            dataProvider.isPlanUseCentralOAI() ? "Y" : "N",
                            dataProvider.isEmailNewsletter() ? "Y" : "N",
                            dataProvider.getHabit().getValue()
                    });
        } catch (DataAccessException ex) {
            int cnt = jdbcTemplate.queryForInt(
                    "select count(1) from data_providers where oai_id=?",
                    new Object[]{dataProvider.getOaiIdentifier()});
            if (cnt > 0) {
                throw new EInfoSzabOperationException(
                        EInfoSzabOperationException.ERROR_OAI_ID_ALREADY_EXIST);
            }

            int cnt2 = jdbcTemplate.queryForInt(
                    "select count(1) from data_providers where name=?",
                    new Object[]{dataProvider.getInstitution().getName()});
            if (cnt2 > 0) {
                throw new EInfoSzabOperationException(
                        EInfoSzabOperationException.ERROR_DP_NAME_ALREADY_EXIST);
            }

            throw ex;
        }

        long dpId = jdbcTemplate.queryForLong("SELECT data_providers.CURRVAL FROM DUAL");
        dataProvider.setId(dpId);

        String dpName = dataProvider.getInstitution().getName();
        dataProvider.getInstitution().getResponsible().setType(Person.TYPE_RESPONSIBLE);
        dataProvider.getInstitution().getResponsible().setSourceId(dpId);
        dataProvider.getInstitution().getResponsible().setSourceName(dpName);
        dataProvider.getInstitution().getResponsible().setUserGroup(Person.USERGROUP_DATAPROVIDER);
        userOperations.createPersonInternal(
                dataProvider.getInstitution().getResponsible());

        if (dataProvider.getInstitution().getContacts() != null) {
            for (Iterator iter = dataProvider.getInstitution().getContacts().iterator();
                 iter.hasNext(); ) {
                Person person = (Person) iter.next();
                person.setType(Person.TYPE_CONTACT);
                person.setSourceId(dpId);
                person.setSourceName(dpName);
                person.setUserGroup(Person.USERGROUP_DATAPROVIDER);
                userOperations.createPersonInternal(person);
            }
        }
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#getDataProvider(long)
     */
    public DataProvider getDataProvider(long id) throws EInfoSzabOperationException {
        if (id <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        Long dpId = new Long(id);
        final DataProvider returnValue = new DataProvider();

        try {
            Map dpMap = jdbcTemplate.queryForMap("select * from data_providers where id = ?", new Object[]{dpId});

            returnValue.setId(Long.parseLong(dpMap.get(COLUMN_DATA_PROVIDER_ID).toString()));
            returnValue.setUrl((String) dpMap.get(COLUMN_DATA_PROVIDER_URL));
            returnValue.setOaiIdentifier((String) dpMap.get(COLUMN_DATA_PROVIDER_OAI_ID));
            returnValue.setPriority(Integer.parseInt(dpMap.get(COLUMN_DATA_PROVIDER_PRIORITY).toString()));
            returnValue.setState((String) dpMap.get(COLUMN_DATA_PROVIDER_STATE));
            returnValue.setLastModified(parseDate((String) dpMap.get(COLUMN_DATA_PROVIDER_LAST_MODIFIED)));
            returnValue.setDisabled("N".equals(dpMap.get(COLUMN_DATA_PROVIDER_ENABLED)));
            returnValue.setSupportedSchemas(getSchemaListFromRepresentation((String) dpMap.get(COLUMN_DATA_PROVIDER_SUPPORTED_SCHEMAS)));
            returnValue.setHarvestChangeSignSupported("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_HARVEST_CHANGESIGN)));
            returnValue.setPlanUseCentralWeb("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_PLAN_USE_WEB)));
            returnValue.setPlanUseCentralOAI("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_PLAN_USE_OAI)));
            returnValue.setPlanUseDefaultEditor("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_PLAN_USE_EDITOR)));
            returnValue.setPlanUseDefaultRepository("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_PLAN_USE_REPO)));
            returnValue.setEmailNewsletter("Y".equals(dpMap.get(COLUMN_DATA_PROVIDER_EMAIL_NEWSLETTER)));

            Object habitObject = dpMap.get(COLUMN_DATA_PROVIDER_HABIT);
            if (habitObject != null) {
                returnValue.setHabit(new DataProviderHabit(Integer.parseInt(habitObject.toString())));
            } else {
                returnValue.setHabit(new DataProviderHabit());
            }

            returnValue.setEmailReportSuccess((String) dpMap.get(COLUMN_DATA_PROVIDER_EMAIL_REPORT_SUCCESS));
            returnValue.setEmailReportTest((String) dpMap.get(COLUMN_DATA_PROVIDER_EMAIL_REPORT_TEST));
            returnValue.setEmailReportUnsucc((String) dpMap.get(COLUMN_DATA_PROVIDER_EMAIL_REPORT_UNSUCC));
            returnValue.setHarvestInterval(Integer.parseInt(dpMap.get(COLUMN_DATA_PROVIDER_HARVEST_INTERVAL).toString()));

            if (dpMap.get(COLUMN_DATA_PROVIDER_USER_TYPE_ID) != null) {
                returnValue.setUserType((String) jdbcTemplate.queryForObject("select user_type from user_types where id=" + dpMap.get(COLUMN_DATA_PROVIDER_USER_TYPE_ID), String.class));
            }

            final Institution inst = new Institution();
            inst.setName((String) dpMap.get("name"));
            inst.setShortName((String) dpMap.get("short_name"));
            inst.setAddress((String) dpMap.get("address"));
            inst.setMailAddress((String) dpMap.get("mail_address"));
            inst.setFaxNumber((String) dpMap.get("fax_num"));
            inst.setPhoneNumber((String) dpMap.get("phone_num"));
            inst.setEmail((String) dpMap.get("email"));
            inst.setWebAddress((String) dpMap.get("web"));
            inst.setLeaderName((String) dpMap.get("leader"));

            jdbcTemplate.query("select phone_num, is_fax from dp_phone where dp_id=?", new Object[]{dpId}, new RowMapper() {
                public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                    String phoneNum = resultSet.getString("phone_num");
                    boolean isFax = resultSet.getBoolean("is_fax");
                    if (isFax) {
                        inst.addFax(phoneNum);
                    } else {
                        inst.addPhone(phoneNum);
                    }
                    return null;
                }
            });

            returnValue.setInstitution(inst);

            if (dpMap.get("harvest_id") != null) {
                Long harvestId = new Long(dpMap.get("harvest_id").toString());
                jdbcTemplate.query("SELECT id, dp_id, start_time, end_time, new_num, modified_num, " + "deleted_num, error_num, success FROM harvests where id=?", new Object[]{harvestId},
                        new ResultSetExtractor() {
                            public Object extractData(ResultSet rs) throws SQLException, DataAccessException {
                                if (rs.next()) {
                                    returnValue.setLastGoodHarvest(createHarvestFromResultSet(rs));
                                    returnValue.getLastGoodHarvest().setDataProviderName(returnValue.getInstitution().getName());
                                }

                                return null;
                            }
                        });
            }

            jdbcTemplate.query("SELECT login_name, name, assignment, email, fax_num, phone_num, web, type " + "FROM users where dp_id=?", new Object[]{dpId},
                    new ResultSetExtractor() {
                        public Object extractData(ResultSet rs) throws SQLException, DataAccessException {
                            while (rs.next()) {
                                Person person = new Person();
                                person.setLoginName(rs.getString("login_name"));
                                person.setName(rs.getString("name"));
                                person.setAssignment(rs.getString("assignment"));
                                person.setEmail(rs.getString("email"));
                                person.setFaxNumber(rs.getString("fax_num"));
                                person.setPhoneNumber(rs.getString("phone_num"));
                                person.setWebAddress(rs.getString("web"));
                                person.setType(rs.getString("type"));
                                person.setSourceId(returnValue.getId());
                                person.setSourceName(returnValue.getInstitution().getName());
                                person.setUserGroup(Person.USERGROUP_DATAPROVIDER);

                                if (Person.TYPE_RESPONSIBLE.equals(person.getType())) {
                                    returnValue.getInstitution().setResponsible(person);
                                } else {
                                    returnValue.getInstitution().addContact(person);
                                }
                            }

                            return null;
                        }

                    });
        } catch (IncorrectResultSizeDataAccessException ex) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID, ex);
        }

        return returnValue;
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#deleteDataProvider(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public void deleteDataProvider(DataProvider dataProvider) throws EInfoSzabOperationException {
        long id = dataProvider.getId();
        if (id <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        String deletedName = dataProvider.guessDeletedName();
        // jdbcTemplate.update("delete from data_providers where id=?", new Object[]{new Long(id)});
        jdbcTemplate.update("update data_providers set name=?, state=? where id=?", new Object[]{deletedName, DataProvider.STATE_DELETED, new Long(id)});
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#saveDataProviderChanges(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public void saveDataProviderChanges(DataProvider provider) throws EInfoSzabOperationException {
        if (provider.getId() <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        Long userTypeId = null;
        if (provider.getUserType() != null) {
            userTypeId = getUserTypeId(provider.getUserType());
        }

        provider.setLastModified(new Date());

        jdbcTemplate.update("update data_providers set url=?, harvest_interval=?, " +
                "harvest_changesign=?, priority=?, name=?, short_name=?, " +
                "leader=?, address=?, mail_address=?, phone_num=?, fax_num=?, web=?, email=?, " +
                "supported_schemas=?, user_type_id=?, plan_use_repo=?, plan_use_editor=?, " +
                "plan_use_centralweb=?, plan_use_centraloai=?, email_report_success=?, email_report_unsucc=?, " +
                "email_report_test=?, email_newsletter=?, habit=?, last_modified=? where id=?",
                new Object[]{
                        provider.getUrl(),
                        new Integer(provider.getHarvestInterval()),
                        provider.isHarvestChangeSignSupported() ? "Y" : "N",
                        new Integer(provider.getPriority()),
                        provider.getInstitution().getName(),
                        provider.getInstitution().getShortName(),
                        provider.getInstitution().getLeaderName(),
                        provider.getInstitution().getAddress(),
                        provider.getInstitution().getMailAddress(),
                        provider.getInstitution().getPhoneNumber(),
                        provider.getInstitution().getFaxNumber(),
                        provider.getInstitution().getWebAddress(),
                        provider.getInstitution().getEmail(),
                        getSchemaRepresentation(provider.getSupportedSchemas()),
                        userTypeId,
                        provider.isPlanUseDefaultRepository() ? "Y" : "N",
                        provider.isPlanUseDefaultEditor() ? "Y" : "N",
                        provider.isPlanUseCentralWeb() ? "Y" : "N",
                        provider.isPlanUseCentralOAI() ? "Y" : "N",
                        provider.getEmailReportSuccess(),
                        provider.getEmailReportUnsucc(),
                        provider.getEmailReportTest(),
                        provider.isEmailNewsletter() ? "Y" : "N",
                        provider.getHabit().getValue(),
                        formatDate(provider.getLastModified()),
                        new Long(provider.getId())
                });

        userOperations.savePersonInternal(provider.getInstitution().getResponsible());
        StringBuffer updateBuf = new StringBuffer();
        updateBuf.append("DELETE FROM users WHERE dp_id=? AND login_name NOT IN ('");
        updateBuf.append(provider.getInstitution().getResponsible().getLoginName());
        updateBuf.append('\'');

        if (provider.getInstitution().getContacts() != null) {
            long dpId = provider.getId();
            String dpName = provider.getInstitution().getName();

            for (Iterator iter = provider.getInstitution().getContacts().iterator();
                 iter.hasNext(); ) {
                Person person = (Person) iter.next();
                if (person.getSourceId() > 0) {
                    userOperations.savePersonInternal(person);
                } else {
                    person.setType(Person.TYPE_CONTACT);
                    person.setUserGroup(Person.USERGROUP_DATAPROVIDER);
                    person.setSourceId(dpId);
                    person.setSourceName(dpName);
                    try {
                        userOperations.createPersonInternal(person);
                    } catch (EInfoSzabOperationException ex) {
                        person.setSourceId(0);
                        throw ex;
                    }
                }
                updateBuf.append(", \'");
                updateBuf.append(person.getLoginName());
                updateBuf.append('\'');
            }
        }

        updateBuf.append(')');

        jdbcTemplate.update(updateBuf.toString(), new Object[]{new Long(provider.getId())});
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#sendActivityRequest(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public void sendActivityRequest(DataProvider provider) throws EInfoSzabOperationException {
        if (provider.getId() <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        boolean canSendRequest = !provider.isDisabled() && DataProvider.STATE_REGISTERED.equals(provider.getState());
        if (canSendRequest) {
            if (!provider.getHabit().isUsingCentralOAI()) { // if use central web doesn't need test harvest
                int successfulTestCount = jdbcTemplate.queryForInt("select count(1) from test_harvests where dp_id=? and success='Y'", new Object[]{new Long(provider.getId())});
                canSendRequest = successfulTestCount > 0;
            }
        }

        if (canSendRequest) {
            provider.setLastModified(new Date());
            jdbcTemplate.update("update data_providers set state=?, last_modified=? where id=?",
                    new Object[]{
                            DataProvider.STATE_WAIT_FOR_ACTIVATION,
                            formatDate(provider.getLastModified()),
                            new Long(provider.getId())
                    }
            );
            provider.setState(DataProvider.STATE_WAIT_FOR_ACTIVATION);
        } else {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_ACTIVITYREQUEST_CANNOT_SEND);
        }
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#sendDeleteRequest(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public void sendDeleteRequest(DataProvider provider) throws EInfoSzabOperationException {
        if (provider.getId() <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        boolean canSendRequest = !provider.isDisabled() && (DataProvider.STATE_REGISTERED.equals(provider.getState()) || DataProvider.STATE_ACTIVE.equals(provider.getState()));

        if (canSendRequest) {
            provider.setLastModified(new Date());
            String newState = DataProvider.STATE_REGISTERED.equals(provider.getState()) ? DataProvider.STATE_WAIT_FOR_DELETE_REG : DataProvider.STATE_WAIT_FOR_DELETE;
            jdbcTemplate.update("update data_providers set state=?, last_modified=? where id=?",
                    new Object[]{
                            newState,
                            formatDate(provider.getLastModified()),
                            new Long(provider.getId())
                    }
            );
            provider.setState(newState);
        } else {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DELETEREQUEST_CANNOT_SEND);
        }
    }

    public String getOaiRepositoryConfigurationXml(DataProvider provider) throws EInfoSzabOperationException {
        if (provider.getId() <= 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DATAPROVIDER_ID_NOT_VALID);
        }

        StringBuffer buf = new StringBuffer(1024);

        buf.append(XML_HEADER);
        buf.append(XML_ROOTSTART);
        buf.append("<identification>\n");
        buf.append("  <repositoryName>");
        buf.append(provider.getInstitution().getName());
        buf.append("</repositoryName>\n");
        buf.append("  <repositoryId>");
        buf.append(provider.getOaiIdentifier());
        buf.append("</repositoryId>\n");
        buf.append("  <baseURL>");
        buf.append(provider.getUrl());
        buf.append("</baseURL>\n");
        buf.append("  <protocolVersion>2.0</protocolVersion>\n");
        buf.append("  <adminEmail>");
        if (provider.getInstitution() != null && provider.getInstitution().getResponsible() != null) {
            Person responsible = provider.getInstitution().getResponsible();
            List emails = getUserEmails(responsible.getLoginName());
            if (emails != null && emails.size() > 0) {
                buf.append(emails.get(0));
            }
        }
        buf.append("</adminEmail>\n");
        buf.append("</identification>\n\n");
        buf.append("<modificationReport>\n");
        buf.append("  <url>");
        buf.append(modificationUrl);
        buf.append("</url>\n");
        buf.append("  <identifierParamName>oaiid</identifierParamName>\n");
        buf.append("  <schemaListParamName>schema</schemaListParamName>\n");
        buf.append("  <mode>GET</mode>\n");
        buf.append("</modificationReport>\n\n");
        if (needIpFilter && harvesterIpList != null && harvesterIpList.size() > 0) {
            buf.append("<allowHarvestsFrom>\n");
            for (Iterator iter = harvesterIpList.iterator(); iter.hasNext(); ) {
                buf.append("  <ip>");
                buf.append(iter.next());
                buf.append("</ip>\n");
            }
            buf.append("</allowHarvestsFrom>\n\n");
        }
        if (provider.getSupportedSchemas() != null) {
            buf.append("<metadataFormats>\n");
            for (Iterator iter = provider.getSupportedSchemas().iterator(); iter.hasNext(); ) {
                Schema schema = (Schema) iter.next();
                buf.append("  <format name=\"");
                buf.append(schema.getPrefix());
                buf.append("\">\n");
                buf.append("    <schema>");
                buf.append(schema.getLocation());
                buf.append("</schema>\n");
                buf.append("    <namespace>");
                buf.append(schema.getSchemaId());
                buf.append("</namespace>\n");
                buf.append("    <rootElementName>");
                buf.append(schema.getRootElementName());
                buf.append("</rootElementName>\n");
                buf.append("  </format>\n");

            }
            buf.append("</metadataFormats>\n\n");
        }
        buf.append(XML_ROOTEND);
        return buf.toString();
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#getSupportedSchemas()
     */
    public List getSupportedSchemas() {
        if (supportedSchemas == null) {
            supportedSchemas = new LinkedList();
            jdbcTemplate.query("SELECT id, schema_id, schema_name, schema_desc, schema_loc, " +
                    "schema_prefix, root_element_name FROM schemas WHERE type='S' order by id asc",
                    new ResultSetExtractor() {
                        public Object extractData(ResultSet rs) throws SQLException, DataAccessException {
                            while (rs.next()) {
                                Schema schema = new Schema();

                                schema.setId(rs.getLong("id"));
                                schema.setSchemaId(rs.getString("schema_id"));
                                schema.setName(rs.getString("schema_name"));
                                schema.setDescription(rs.getString("schema_desc"));
                                schema.setLocation(rs.getString("schema_loc"));
                                schema.setPrefix(rs.getString("schema_prefix"));
                                schema.setRootElementName(rs.getString("root_element_name"));

                                supportedSchemas.add(schema);
                            }
                            return null;
                        }
                    });
        }

        //return Collections.unmodifiableList(supportedSchemas);
        return supportedSchemas;
    }

    public List getDefaultSchemas() {
        if (defaultSchemas == null) {
            if (defaultSchemaIds == null) {
                defaultSchemas = Collections.EMPTY_LIST;
            } else {
                defaultSchemas = new LinkedList();
                List schemas = getSupportedSchemas();

                Iterator i = schemas.iterator();
                while (i.hasNext()) {
                    Schema schema = (Schema) i.next();
                    if (defaultSchemaIds.contains(schema.getSchemaId())) {
                        defaultSchemas.add(schema);
                    }
                }
            }
        }

        //return Collections.unmodifiableList(defaultSchemas);
        return defaultSchemas;
    }

    public void setDefaultSchemaIds(Set defaultSchemaIds) {
        this.defaultSchemaIds = defaultSchemaIds;
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#getUserTypes()
     */
    public List getUserTypes() {
        List returnValue = (List) jdbcTemplate.query("select id, user_type from user_types",
                new ResultSetExtractor() {
                    public Object extractData(ResultSet rs) throws SQLException, DataAccessException {
                        List list = new LinkedList();

                        while (rs.next()) {
                            UserType type = new UserType();
                            type.setId(rs.getLong("id"));
                            type.setName(rs.getString("user_type"));
                            list.add(type);
                        }

                        return list;
                    }
                });

        Collections.sort(returnValue);
        return returnValue;
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#getActiveRecordCount(hu.scriptum.einfoszab.datamodel.DataProvider)
     */
    public int getActiveRecordCount(DataProvider provider) {
        int returnValue = 0;

        String providerWhere = "";
        if (provider != null) {
            providerWhere = " AND dp_id=" + provider.getId();
        }

        returnValue = jdbcTemplate.queryForInt("SELECT count(1) FROM xmls WHERE state='A'" + providerWhere);

        return returnValue;
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#addUserType(String)
     */
    public void addUserType(String type) throws EInfoSzabOperationException {
        int count = jdbcTemplate.queryForInt("select count(1) from user_types where user_type=?", new Object[]{type});
        if (count > 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_USERTYPE_ALREADY_EXIST);
        }

        jdbcTemplate.update("insert into user_types(user_type) values(?)", new Object[]{type});
    }

    /**
     * @throws org.springframework.dao.DataAccessException
     *          if a database exception occurs
     * @see hu.scriptum.einfoszab.api.DataProviderOperations#removeUserType(long)
     */
    public void removeUserType(long id) throws EInfoSzabOperationException {
        int count = jdbcTemplate.queryForInt("select count(1) from data_providers " + "where user_type_id=" + id);
        if (count > 0) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_USERTYPE_CANNOT_DELETE);
        }

        jdbcTemplate.update("delete from user_types where id=" + id);
    }

    public long getDataProviderId(String name) throws EInfoSzabOperationException {
        long returnValue = 0;

        try {
            returnValue = jdbcTemplate.queryForLong("select id from data_providers where name=?", new Object[]{name});
        } catch (IncorrectResultSizeDataAccessException ex) {
            throw new EInfoSzabOperationException(EInfoSzabOperationException.ERROR_DP_NAME_NOT_UNIQUE);
        }

        return returnValue;
    }

    /**
     * @return adatgazda neve (ami többnyire azonos az intzémény nevével)
     */
    public String getDataProviderName(long dataProviderId) {
        String sql = "select name from data_providers where id=?";
        return (String) jdbcTemplate.query(sql, new Object[]{dataProviderId}, new ResultSetExtractor() {
            public Object extractData(ResultSet resultSet) throws SQLException, DataAccessException {
                String dpName = "";
                if (resultSet.next()) {
                    dpName = resultSet.getString(COLUMN_DATA_PROVIDER_NAME);
                }
                return dpName;
            }
        });

    }

    public String getDataProviderOaiId(long dataProviderId) {
        String sql = "select oai_id from data_providers where id=?";
        return (String) jdbcTemplate.query(sql, new Object[]{dataProviderId}, new ResultSetExtractor() {
            public Object extractData(ResultSet resultSet) throws SQLException, DataAccessException {
                String oaiId = "";
                if (resultSet.next()) {
                    oaiId = resultSet.getString(COLUMN_DATA_PROVIDER_OAI_ID);
                }
                return oaiId;
            }
        });
    }

    public List<InstitutionType> getInstitutionTypes() {

        String sql = "select id, type, description from institution_type";
        List institutionTypes = jdbcTemplate.query(sql, new InstitutionTypeRowMapper());

        LOGGER.debug("institutionTypes: " + institutionTypes);

        return institutionTypes;

    }

    public List getDPPhones(long dataProviderId) {
        return jdbcTemplate.query("select id, phone_num, is_fax from dp_phone where dp_id=? order by id", new Object[]{dataProviderId}, new PhoneRowMapper());
    }

    public List getUserPhones(String loginName) {
        return jdbcTemplate.query("select id, phone_num, is_fax from user_phone where login_name=?", new Object[]{loginName}, new PhoneRowMapper());
    }

    public List getUserEmails(String loginName) {
        return jdbcTemplate.query("select email from user_email where login_name=?", new Object[]{loginName}, new RowMapper() {
            public Object mapRow(ResultSet resultSet, int i) throws SQLException {
                return resultSet.getString(COLUMN_DATA_PROVIDER_EMAIL);
            }
        });
    }

    public List getInstitutionTypes(long dataProviderId) {
        /*String sql = "select it.type from INSTITUTION_TYPE it " +
                          "join DP_INST_TYPE dp_it on it.ID=dp_it.INST_TYPE_ID " +
                          "join DATA_PROVIDERS dp on dp.ID=dp_it.DP_ID " +
                          "where dp.id=?;"; */

        String sql = "select it.type from INSTITUTION_TYPE it, " +
                "DP_INST_TYPE dp_it,  DATA_PROVIDERS dp " +
                "where it.ID=dp_it.INST_TYPE_ID and dp.ID=dp_it.DP_ID and dp.id=?";

        return jdbcTemplate.queryForList(sql, new Object[]{dataProviderId}, String.class);
    }

    public int updateInstitutionData(DataProvider dp) {
        Institution inst = dp.getInstitution();
        LOGGER.debug("Institution will be update for this dp id: " + dp.getId());

        String sql = "update data_providers set name=?, short_name=?, oai_id=?, " +
                "address=?, mail_address=?, web=?, email=?, leader=?, user_type_id=?, " +
                "last_modified=? where id=?";

        int affectedRows = jdbcTemplate.update(sql, new Object[]{
                inst.getName(),
                inst.getShortName(),
                dp.getOaiIdentifier(),
                inst.getAddress(),
                inst.getMailAddress(),
                inst.getWebAddress(),
                inst.getEmail(),
                inst.getLeaderName(),
                getUserTypeId(dp.getUserType()),
                formatDate(dp.getLastModified()),
                dp.getId()
        });

        int deleted = deleteDPPhones(dp.getId());
        LOGGER.debug("before phones and faxes update: " + deleted + " old entries were deleted");
        addDPPhones(dp);

        deleted = deleteInstitutionTypes(dp.getId());
        LOGGER.debug("before institution types update: " + deleted + " old entries were deleted");
        addInstitutionTypes(dp);

        Address address = inst.getNewAddress();
        LOGGER.debug("update address: " + address);
        long addressId = jdbcTemplate.queryForLong("select address_id from data_providers where id=?", new Object[]{dp.getId()});

        jdbcTemplate.update("update dp_address da set zipcode_settlement_id=?, address=?, longitude=?, latitude=? where da.id=?", new Object[]{address.getZipCode().getId(), address.getAddress(), address.getLongitude(), address.getLatitude(), addressId});
//        if (address.getId() == null) {
//            affectedRows = jdbcTemplate.update("insert into dp_address(zipcode_settlement_id, address, longitude, latitude) values (?, ?, ?, ?)", new Object[]{address.getZipCode().getId(), address.getAddress(), address.getLongitude(), address.getLatitude()});
//            jdbcTemplate.update("update data_providers set address_id=(select dp_address.currval from dual) where id=?", new Object[]{dp.getId()});
//        } else {
//            long addressId = jdbcTemplate.queryForLong("select address_id from data_providers where id=?", new Object[]{dp.getId()});
//            jdbcTemplate.update("update dp_address da set zipcode_settlement_id=?, address=?, longitude=?, latitude=? where da.id=?", new Object[]{address.getZipCode().getId(), address.getAddress(), address.getLongitude(), address.getLatitude(), addressId});
//        }

        return affectedRows;
    }

    public int updateOaiData(DataProvider dp) {

        String sql = "update data_providers set oai_id=?, url=?, harvest_interval=?, harvest_changesign=?, supported_schemas=?, " +
                "plan_use_repo=?, plan_use_editor=?, plan_use_centralweb=?, plan_use_centraloai=?, email_report_success=?, email_report_unsucc=?, " +
                "email_report_test=?, email_newsletter=?, habit=?, last_modified=? where id=?";

        return jdbcTemplate.update(sql, new Object[]{
                dp.getOaiIdentifier(),
                dp.getUrl(),
                dp.getHarvestInterval(),
                dp.isHarvestChangeSignSupported() ? "Y" : "N",
                getSchemaRepresentation(dp.getSupportedSchemas()),
                dp.isPlanUseDefaultRepository() ? "Y" : "N",
                dp.isPlanUseDefaultEditor() ? "Y" : "N",
                dp.isPlanUseCentralWeb() ? "Y" : "N",
                dp.isPlanUseCentralOAI() ? "Y" : "N",
                dp.getEmailReportSuccess(),
                dp.getEmailReportUnsucc(),
                dp.getEmailReportTest(),
                dp.isEmailNewsletter() ? "Y" : "N",
                dp.getHabit().getValue(),
                formatDate(dp.getLastModified()),
                new Long(dp.getId())

        });
    }

    public int updateHarvestInterval(long dataProviderId, int harvestInterval) {
        String sql = "update data_providers set harvest_interval=? where id=?";
        return jdbcTemplate.update(sql, new Object[]{harvestInterval, dataProviderId});
    }

    protected String generatePossibleOaiId(String shortName) {
        StringBuffer buf = new StringBuffer(shortName.length());
        for (int i = 0; i < shortName.length(); i++) {
            char c = shortName.charAt(i);
            if (Character.isLetterOrDigit(c)) {
                buf.append(c);
            } else {
                LOGGER.debug("Character '0x" + Integer.toHexString(c) + "' will be cut off from generated OAI id... ");
            }
        }

        LOGGER.debug("Generated OAI identified BEFORE normalizing: " + buf.toString());
        return NormalizerUtilities.normalizeString(buf.toString()).toLowerCase();
    }


    public String getHarvestURL(long dataproviderId) {
        final String[] result = new String[1];
        jdbcTemplate.query("select url from dp_url where dp_id=? and url_type=?", new Object[]{dataproviderId, 'H'}, new RowCallbackHandler() {
            public void processRow(ResultSet resultSet) throws SQLException {
                result[0] = resultSet.getString(COLUMN_DATA_PROVIDER_URL);
            }
        });
        return result[0];
    }

    public Address getAddress(final long dataProviderId) {
        Address address = (Address) jdbcTemplate.query(QUERY_FULL_ADDRESS, new Object[]{dataProviderId}, new ResultSetExtractor() {
            public Object extractData(ResultSet resultSet) throws SQLException, DataAccessException {
                if (resultSet.next()) {

                    Object addressIdObject = resultSet.getObject("addressId");
                    Long addressId = null;
                    if (addressIdObject instanceof BigDecimal) {
                        addressId = ((BigDecimal) addressIdObject).longValue();
                    }

                    String addr = resultSet.getString("addressAddress");

                    Object zipCodeSettlementIdObject = resultSet.getObject("zipcode_settlement_id");
                    Long zipCodeId = null;
                    if (zipCodeSettlementIdObject instanceof BigDecimal) {
                        zipCodeId = ((BigDecimal) zipCodeSettlementIdObject).longValue();
                    }

                    String zipcode = resultSet.getString("zipcode");

                    Object settlementIdObject = resultSet.getObject("settlementId");
                    Long settlementId = null;
                    if (settlementIdObject instanceof BigDecimal) {
                        settlementId = ((BigDecimal) settlementIdObject).longValue();
                    }

                    LOGGER.debug("getAddress(): dataProviderId=" + dataProviderId + ", addressId=" + addressId + ", zipCodeId=" + zipCodeId + ", settlementId=" + settlementId);

                    String settlement = resultSet.getString("settlementName");
                    Double latitude = resultSet.getDouble("addressLatitude");
                    if (resultSet.wasNull()) {
                        latitude = null;
                    }
                    Double longitude = resultSet.getDouble("addressLongitude");
                    if (resultSet.wasNull()) {
                        longitude = null;
                    }

                    Address address = new Address();
                    address.setId(addressId);
                    address.setAddress(addr);
                    address.setLatitude(latitude);
                    address.setLongitude(longitude);

                    ZipCode zipCode = new ZipCode();
                    zipCode.setId(zipCodeId);
                    zipCode.setZipCode(zipcode);

                    Settlement settlementObject = new Settlement();
                    settlementObject.setId(settlementId);
                    settlementObject.setName(settlement);
                    settlementObject.setLongitude(resultSet.getDouble("settlementLongitude"));
                    settlementObject.setLatitude(resultSet.getDouble("settlementLatitude"));

                    SubRegion subRegion = new SubRegion();
                    subRegion.setId(resultSet.getLong("subRegionId"));
                    subRegion.setName(resultSet.getString("subRegionName"));
                    subRegion.setLongitude(resultSet.getDouble("subRegionLongitude"));
                    subRegion.setLatitude(resultSet.getDouble("subRegionLatitude"));

                    County county = new County();
                    county.setId(resultSet.getLong("countyId"));
                    county.setName(resultSet.getString("countyName"));
                    county.setLongitude(resultSet.getDouble("countyLongitude"));
                    county.setLatitude(resultSet.getDouble("countyLatitude"));

                    Region region = new Region();
                    region.setId(resultSet.getLong("regionId"));
                    region.setName(resultSet.getString("regionName"));
                    region.setLongitude(resultSet.getDouble("regionLongitude"));
                    region.setLatitude(resultSet.getDouble("regionLatitude"));

                    county.setRegion(region);

                    subRegion.setCounty(county);

                    settlementObject.setSubRegion(subRegion);

                    zipCode.setSettlement(settlementObject);

                    address.setZipCode(zipCode);

                    return address;
                } else {
                    return null;
                }
            }
        });
        return address;
    }

    public ZipCode getZipCode(long zipCodeSettlementId) {
        LOGGER.debug("getZipCode() CALLED, params: zipCodeSettlementId=" + zipCodeSettlementId);
        ZipCode zipCode = null;
        try {
            Object o = jdbcTemplate.queryForObject(QUERY_ZIPCODE_BY_ID, new Object[]{zipCodeSettlementId}, new ZipCodeMapper());
            if (o instanceof ZipCode) {
                zipCode = (ZipCode) o;
            }
        } catch (IncorrectResultSizeDataAccessException e) {
            LOGGER.warn("zipCodeSettlementId=" + zipCodeSettlementId + ", " + e.getMessage());
        }

        LOGGER.debug("getZipCode() FINISHED, returning=" + zipCode);
        return zipCode;
    }

    public List<ZipCode> getZipCodes() {
        LOGGER.debug("getZipCodes() CALLED");

        List<ZipCode> zipCodeList = new LinkedList<ZipCode>();

        Collection queryResult = jdbcTemplate.query(QUERY_ZIPCODES_WITH_SETTLEMENT + ORDER_BY_ZIPCODE_TAIL, new ZipCodeMapper());
        for (Object o : queryResult) {
            if (o instanceof ZipCode) {
                zipCodeList.add((ZipCode) o);
            }
        }

        LOGGER.debug("getZipCodes() FINISHED");
        return zipCodeList;
    }

    public List<ZipCode> getZipCodesByPattern(String pattern) {
        if (LOGGER.isDebugEnabled()) {
            LOGGER.debug("getZipCodesByPattern() CALLED, params: pattern=" + pattern);
        }

        if (pattern == null || "".equals(pattern.trim())) {
            return getZipCodes();
        }

        List<ZipCode> zipCodeList = new LinkedList<ZipCode>();

        String patternParam = new StringBuilder().append("%").append(pattern.toLowerCase()).append("%").toString();
        Collection queryResult = jdbcTemplate.query(QUERY_ZIPCODES_WITH_SETTLEMENT_BY_PATTERN, new Object[]{patternParam, patternParam}, new ZipCodeMapper());
        for (Object o : queryResult) {
            if (o instanceof ZipCode) {
                zipCodeList.add((ZipCode) o);
            }
        }

        LOGGER.debug("getZipCodesByPattern() FINISHED");
        return zipCodeList;
    }

    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public void setUserOperations(RdbmsUserOperations userOperations) {
        this.userOperations = userOperations;
    }

    public void setHarvesterIpList(List harvesterIpList) {
        this.harvesterIpList = harvesterIpList;
    }

    public void setModificationUrl(String modificationUrl) {
        this.modificationUrl = modificationUrl;
    }

    public void setNeedIpFilter(boolean needIpFilter) {
        this.needIpFilter = needIpFilter;
    }


    protected Date getDateFromString(final String dateStr) {
        Date returnValue = null;

        try {
            returnValue = (dateStr == null ? null : formatter.parse(dateStr));
        } catch (ParseException ex) {
            LOGGER.warn("Exception in parsing date: " + dateStr, ex);
        }

        return returnValue;
    }

    protected Harvest createHarvestFromResultSet(ResultSet rs) throws SQLException {
        Harvest harvest = new Harvest();

        harvest.setId(rs.getLong("id"));
        harvest.setDataProviderId(rs.getLong("dp_id"));
        harvest.setStartTime(getDateFromString(rs.getString("start_time")));
        harvest.setEndTime(getDateFromString(rs.getString("end_time")));
        harvest.setRecordsAdded(rs.getInt("new_num"));
        harvest.setRecordsModified(rs.getInt("modified_num"));
        harvest.setRecordsDeleted(rs.getInt("deleted_num"));
        harvest.setNumberOfErrors(rs.getInt("error_num"));
        String success = rs.getString("success");
        harvest.setSuccess("Y".equals(success));
        harvest.setProcessed(success == null);

        return harvest;
    }


}
