/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2010.02.17.
 * Time: 10:06:55
 */
package hu.kozadat.search.persistence.core;

import org.springframework.jdbc.core.JdbcOperations;
import org.springframework.jdbc.core.RowMapper;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;

public class RdbmsStatisticOperations {

    private JdbcOperations jdbcTemplate;

    public static final String TABLE_SEARCH_STATISTIC = "search_statistic";
    public static final String COLUMN_SEARCH_STATISTIC_ID = "id";
    public static final String COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION = "search_expression";
    public static final String COLUMN_SEARCH_STATISTIC_COUNT = "expression_count";

    public static final String UPDATE_SEARCH_EXPRESSION =
            "UPDATE " +
                    TABLE_SEARCH_STATISTIC + " " +
                    "SET " +
                    COLUMN_SEARCH_STATISTIC_COUNT + "=" + COLUMN_SEARCH_STATISTIC_COUNT + "+1" +
                    "WHERE " +
                    COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION + " LIKE ?";

    public static final String INSERT_SEARCH_EXPRESSION =
            "INSERT INTO " +
                    TABLE_SEARCH_STATISTIC +
                    " (" +
                    COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION +
                    ") " +
                    "VALUES (?)";

    public static final String QUERY_SEARCH_EXPRESSION =
            "SELECT " +
                    COLUMN_SEARCH_STATISTIC_ID + ", " +
                    COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION + ", " +
                    COLUMN_SEARCH_STATISTIC_COUNT + " " +
                    "FROM " +
                    TABLE_SEARCH_STATISTIC + " " +
                    "WHERE " +
                    COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION + " NOT LIKE '%(%)%' " +
                    "ORDER BY " +
                    COLUMN_SEARCH_STATISTIC_COUNT + " DESC " +
                    " LIMIT ?";

    private RowMapper rowMapper = new RowMapper() {

        public Object mapRow(ResultSet resultSet, int i) throws SQLException {
            return resultSet.getString(COLUMN_SEARCH_STATISTIC_SEARCH_EXPRESSION);
        }
    };


    public void setJdbcTemplate(JdbcOperations jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public void updateOrInsertSearchQuery(String searchQuery) {
        int rowsUpdated = jdbcTemplate.update(UPDATE_SEARCH_EXPRESSION, new Object[]{searchQuery});
        if (rowsUpdated == 0) {
            jdbcTemplate.update(INSERT_SEARCH_EXPRESSION, new Object[]{searchQuery});
        }
    }

    public List<String> getPrevalentSearchExpressions(int count) {
        List resultList = jdbcTemplate.query(QUERY_SEARCH_EXPRESSION, new Object[]{count}, rowMapper);
        List<String> prevalentSearchExpressions = new LinkedList<String>();
        for (Object o : resultList) {
            if (o instanceof String) {
                prevalentSearchExpressions.add((String) o);
            }
        }
        return prevalentSearchExpressions;
    }

    public List<String> getRandomPrevalentSearchExpressions(int count, int randomInterval) {
        List resultList = jdbcTemplate.query(QUERY_SEARCH_EXPRESSION, new Object[]{randomInterval}, rowMapper);
        List<String> randomPrevalentSearchExpressions = new LinkedList<String>();
        if (resultList != null) {
            if (resultList.size() > count) {
                Random random = new Random();
                for (int i = 0; i < count; i++) {
                    int index = random.nextInt(resultList.size());
                    Object o = resultList.get(index);
                    if (o instanceof String) {
                        resultList.remove(o);
                        randomPrevalentSearchExpressions.add((String) o);
                    }
                }
            } else {
                for (Object o : resultList) {
                    if (o instanceof String) {
                        randomPrevalentSearchExpressions.add((String) o);
                    }
                }
            }
        }
        return randomPrevalentSearchExpressions;
    }
}
