package hu.kozadat.search.presentation.autocomplete;

import hu.scriptum.einfoszab.rdbms.DataModelElement;
import hu.scriptum.web.spring.input.TyperListDataModel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Spring kontroller, amely segitsegevel a script.aculo.us aktualis (1.5) valtozataval
 * lehetosegunk van gepeles Proxy-szeru beviteli komponens keszitesere.
 *
 * @author Csaba Horvath, Scriptum Inc.
 * @version $Id$
 */
@Deprecated
public class TyperListController extends AbstractController {

    private static final Logger LOGGER = LoggerFactory.getLogger(TyperListController.class);

    private TyperListDataModel dataModel;
    private String textFieldName;
    private int maxListSize;
    private String clientEncoding;
    private String cssUlClass;
    private String cssLiPartialClass;
    private String cssLiFullClass;
    private String cssLiActiveClass;
    private String cssLiRegisteredClass;
    private String cssLiOtherClass;
    private String cssPartialMessageClass;
    private String partialMessage;

    public TyperListController() {
        // Alapertelmezett ertekek beallitasa
        textFieldName = "prefix";
        maxListSize = 5;
        clientEncoding = "UTF-8";
        cssUlClass = "typerList";
        cssLiFullClass = "typerListFull";
        cssLiPartialClass = "typerListPartial";
        cssLiActiveClass = "active";
        cssLiRegisteredClass = "registered";
        cssLiOtherClass = "other";
        cssPartialMessageClass = "partialMessage";
        partialMessage = "- még $remain találat";
    }

    /**
     * Beallitja azt az adatmodellt, amelybol a prefix segitsegevel a lehetseges
     * ertekek el lesznek kerve a megjelenitendo listahoz. <b>Beallitasa kotelezo</b>.
     *
     * @param dataModel data model for suggesting values
     */
    public void setDataModel(TyperListDataModel dataModel) {
        if (dataModel == null) {
            throw new IllegalArgumentException("cannot be null");
        }

        this.dataModel = dataModel;
    }

    /**
     * Visszaadja a lehetseges ertekek elkeresere hasznalt adatmodellt.
     *
     * @return data model for suggesting values
     */
    public TyperListDataModel getDataModel() {
        return dataModel;
    }

    /**
     * Beallitja annak a form mezonek a nevet, amelytol a prefixet el kell kerni.
     * Ha nem allitunk semmit, alapertelmezett ertek a <code>prefix</code>.
     *
     * @param textFieldName name of form text field to get prefix from
     */
    public void setTextFieldName(String textFieldName) {
        if (textFieldName == null || textFieldName.trim().length() == 0) {
            throw new IllegalArgumentException("cannot be empty");
        }

        this.textFieldName = textFieldName;
    }

    /**
     * Visszaadja annak a form mezonek a nevet, amelytol a prefixet el kell kerni.
     *
     * @return name of form text field to get prefix from
     */
    public String getTextFieldName() {
        return textFieldName;
    }

    /**
     * Beallitja, hogy egyszerre legfeljebb hany javaslatot lathatunk a kiegeszitesnel.
     * Amennyiben ennel tobbet ad vissza a modell hivas visszateresi erteke, akkor
     * reszleges lista-megjelenites lesz, egyebkent teljes. Ha nem allitunk semmit,
     * alapertelmezett ertek <code>5</code> lesz. erteke 1 es 50 kozott allithato.
     *
     * @param maxListSize number of items can display in a suggestion list
     */
    public void setMaxListSize(int maxListSize) {
        if (maxListSize < 1 || maxListSize > 50) {
            throw new IllegalArgumentException("must be between 1 and 50: " + maxListSize);
        }

        this.maxListSize = maxListSize;
    }

    /**
     * Visszaadja, hogy egyszerre legfeljebb hany javaslatot lathatunk a kiegeszitesnel.
     *
     * @return number of items can display in a suggestion list
     */
    public int getMaxListSize() {
        return maxListSize;
    }

    /**
     * Beallitja azt a karakterkodolast, ami a kliens bongeszojetol jon. Ezt hasznalja
     * a parameter ertekenek dekodolasara, valamint a visszateresi ertek
     * osszeallitasa soran is. Ha nem allitunk semmit, alapertelmezett ertek
     * <code>UTF-8</code> lesz.
     *
     * @param clientEncoding encoding use on the client browser
     */
    public void setClientEncoding(String clientEncoding) {
        if (clientEncoding == null) {
            throw new IllegalArgumentException("cannot be null");
        }

        // kodolas ellenorzese - lehet, hogy az adott VM nem ismeri
        try {
            "lajos".getBytes(clientEncoding);
        } catch (UnsupportedEncodingException ex) {
            throw new IllegalArgumentException("unsupported encoding: " + clientEncoding);
        }

        this.clientEncoding = clientEncoding;
    }

    /**
     * Visszaadja azt a karakterkodolast, ami a kliens bongeszojetol jon. Ezt hasznalja
     * a parameter ertekenek dekodolasara, valamint a visszateresi ertek
     * osszeallitasa soran is.
     *
     * @return encoding use on the client browser
     */
    public String getClientEncoding() {
        return clientEncoding;
    }

    /**
     * Alapertelmezett erteke <code>typerList</code>.
     *
     * @param cssUlClass UL CSS class name
     */
    public void setCssUlClass(String cssUlClass) {
        this.cssUlClass = cssUlClass;
    }

    public String getCssUlClass() {
        return cssUlClass;
    }

    /**
     * Alapertelmezett erteke <code>typerListFull</code>.
     *
     * @param cssLiFullClass LI CSS class name for full lists
     */
    public void setCssLiFullClass(String cssLiFullClass) {
        this.cssLiFullClass = cssLiFullClass;
    }

    public String getCssLiFullClass() {
        return cssLiFullClass;
    }

    /**
     * Alapertelmezett erteke <code>typerListPartial</code>.
     *
     * @param cssLiPartialClass LI CSS class name for partial lists
     */
    public void setCssLiPartialClass(String cssLiPartialClass) {
        this.cssLiPartialClass = cssLiPartialClass;
    }

    public String getCssLiPartialClass() {
        return cssLiPartialClass;
    }

    /**
     * Alapertelmezett erteke <code>partialMessage</code>.
     *
     * @param cssPartialMessageClass FONT CSS class name for partial list messages
     */
    public void setCssPartialMessageClass(String cssPartialMessageClass) {
        this.cssPartialMessageClass = cssPartialMessageClass;
    }

    public String getCssPartialMessageClass() {
        return cssPartialMessageClass;
    }

    /**
     * Beallitja azt az uzenetet, amit az utolso elem ala irunk ki akkor, ha
     * reszleges listarol van szo.
     *
     * @param partialMessage partial message for lists
     */
    public void setPartialMessage(String partialMessage) {
        this.partialMessage = partialMessage;
    }

    public String getPartialMessage() {
        return partialMessage;
    }

    private void appendCssClassIfNecessary(StringBuffer buffer, String className) {
        appendCssClassIfNecessary(buffer, new String[]{className});
    }

    private void appendCssClassIfNecessary(StringBuffer buffer, String[] classNames) {
        if (classNames != null && classNames.length > 0) {
            buffer.append(" class=\"");
            for (String className : classNames) {
                if (className != null && className.length() > 0) {
                    buffer.append(className).append(" ");
                }
            }
            buffer.append('\"');
        }
    }

    private void appendPartialMessage(StringBuffer buf, int size) {
        String appendString = partialMessage;
        int index = appendString.indexOf("$remain");
        if (index == 0) {
            appendString = "" + (size - maxListSize) + appendString.substring(7);
        } else if (index > 0) {
            appendString = appendString.substring(0, index) + (size - maxListSize) + appendString.substring(index + 7);
        }
        index = appendString.indexOf("$size");
        if (index == 0) {
            appendString = "" + size + appendString.substring(7);
        } else if (index > 0) {
            appendString = appendString.substring(0, index) + size + appendString.substring(index + 7);
        }
        buf.append("<br><font ");
        appendCssClassIfNecessary(buf, cssPartialMessageClass);
        buf.append('>');
        buf.append(appendString);
        buf.append("</font>");
    }

    protected ModelAndView handleRequestInternal(HttpServletRequest request, HttpServletResponse response) throws Exception {
        LOGGER.debug("SearchUI - handleRequestInternal() CALLED");
        String prefix = ServletRequestUtils.getStringParameter(request, textFieldName, "");

        Map items = dataModel.fillResultMap(prefix);
        int size = items.size();
        if (items.containsKey(TyperListDataModel.SIZE_DESCRIPTION_KEY)) {
            size = ((Integer) items.remove(TyperListDataModel.SIZE_DESCRIPTION_KEY)).intValue();
        }
        boolean partialListDisplay = (size > maxListSize);

        StringBuffer buf = new StringBuffer(512);
        buf.append("<ul");
        appendCssClassIfNecessary(buf, cssUlClass);
        buf.append(">");
        int i = 0;
        for (Iterator iter = items.keySet().iterator(); i < maxListSize && iter.hasNext(); ) {
            Object keyObject = iter.next();
            if (keyObject instanceof DataModelElement) {
                DataModelElement key = (DataModelElement) keyObject;

                buf.append("<li");
                List<String> cssClasses = new ArrayList<String>();
                if (partialListDisplay) {
                    cssClasses.add(cssLiPartialClass);
                } else {
                    cssClasses.add(cssLiFullClass);
                }

                if (DataModelElement.ATTRIBUTE_VALUE_TYPE_ACTIVE.equals(key.getAttribute(DataModelElement.ATTRIBUTE_NAME_TYPE))) {
                    cssClasses.add(cssLiActiveClass);
                } else if (DataModelElement.ATTRIBUTE_VALUE_TYPE_REGISTERED.equals(key.getAttribute(DataModelElement.ATTRIBUTE_NAME_TYPE))) {
                    cssClasses.add(cssLiRegisteredClass);
                } else if (DataModelElement.ATTRIBUTE_VALUE_TYPE_OTHER.equals(key.getAttribute(DataModelElement.ATTRIBUTE_NAME_TYPE))) {
                    cssClasses.add(cssLiOtherClass);
                }

                String[] cssClassArray = new String[cssClasses.size()];
                appendCssClassIfNecessary(buf, cssClasses.toArray(cssClassArray));
                buf.append('>');

                buf.append(key.getElementValue());

                // tnagy remark is currently unused
                String remark = (String) items.get(key);
                if (remark != null && remark.length() > 0) {
                    buf.append("<span class=\"informal\"> ");
                    buf.append(remark);

                    if (partialListDisplay) {
                        if (i == (maxListSize - 1) && partialMessage != null) {
                            appendPartialMessage(buf, size);
                        }
                    }
                    buf.append(" </span>");
                } else if (i == (maxListSize - 1) && partialListDisplay && partialMessage != null) {
                    buf.append("<span class=\"informal\"> ");
                    appendPartialMessage(buf, size);
                    buf.append(" </span>");
                }
                buf.append("</li>");
            }
            i++;
        }
        buf.append("</ul>");
        String output = buf.toString();
        response.setContentType("text/html; charset=" + clientEncoding);
        response.setCharacterEncoding(clientEncoding);
        response.setStatus(HttpServletResponse.SC_OK);
        byte[] array = output.getBytes("UTF-8");
        response.setContentLength(array.length);
        response.getOutputStream().write(array);
        response.flushBuffer();
        return null;
    }

    static {
        LOGGER.info("Loading class in SearchUI: " + TyperListController.class);
    }

}
