package hu.kozadat.search.presentation.dataprovider;

import hu.kozadat.search.model.DataProviderDisplayInfo;
import hu.kozadat.search.persistence.DataProviderDisplayInfoOperations;
import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.persistence.core.DataProviderDisplayInfoDAO;
import hu.kozadat.search.presentation.DataProviderMapConstants;
import hu.kozadat.search.presentation.GMIFrameSrcParam;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.service.log.SearchLogger;
import hu.kozadat.search.web.map.MapLayout;
import hu.scriptum.einfoszab.datamodel.Address;
import hu.scriptum.einfoszab.datamodel.DataProvider;
import hu.scriptum.einfoszab.datamodel.SubRegion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Arrays;
import java.util.Map;
import java.util.Set;

@Controller
public class DataProviderDisplayInfoController {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataProviderDisplayInfoController.class);

    private static final String URL_ENCODING = "UTF-8";

    private DataProviderOperations dataProviderOperations;

    private DataProviderDisplayInfoOperations dataProviderDisplayInfoOperations;
    private SearchLogger searchLogger;
    private SearchUIConfiguration searchUIConfiguration;

    private static final String PROTOCOL_HTTP = "http://";
    private static final String REQUEST_HEADER_PARAM_HOST = "Host";

    @Autowired
    public void setSearchLogger(SearchLogger searchLogger) {
        this.searchLogger = searchLogger;
    }

    @Autowired
    public void setDataProviderOperations(DataProviderOperations dataProviderOperations) {
        this.dataProviderOperations = dataProviderOperations;
    }

    @Autowired
    public void setDataProviderDisplayInfoOperations(DataProviderDisplayInfoOperations dataProviderDisplayInfoOperations) {
        this.dataProviderDisplayInfoOperations = dataProviderDisplayInfoOperations;
    }

    protected String getGoogleMapsAPIKey(HttpServletRequest request) {
        Map<String, String> googleMapsAPIKeys = searchUIConfiguration.getGoogleMapsAPIKeyMap();
        Set<String> keys = googleMapsAPIKeys.keySet();
        // if 1 entry, then return
        if (keys.size() == 1) {
            return googleMapsAPIKeys.get(keys.iterator().next());
        }
        String hostKey;
        String host = request.getHeader(REQUEST_HEADER_PARAM_HOST);
        LOGGER.debug("host=" + host + ", in map? " + googleMapsAPIKeys.containsKey(host));
        // search by postfix
        String postFixKey = null;
        for (String key : keys) {
            if (host.lastIndexOf(key) != -1) {
                postFixKey = key;
            }
        }
        // if no match, return first key
        if (postFixKey == null) {
            hostKey = keys.iterator().next();
        } else {
            hostKey = postFixKey;
        }
        return googleMapsAPIKeys.get(hostKey);
    }

    @RequestMapping("/intezmeny/egyeb/adatlap/{externalInstituteId}")
    public ModelAndView showExternalInstituteInfo(HttpServletRequest httpServletRequest, @PathVariable("externalInstituteId") String externalInstituteIdStr) throws Exception {
        String mapServletHost = PROTOCOL_HTTP + httpServletRequest.getHeader(REQUEST_HEADER_PARAM_HOST);
        String mapServletContext = httpServletRequest.getContextPath();
        String mapServletPath = "mapservlet.kml";

        LOGGER.debug("mapServletHost=" + mapServletHost + ", mapServletContext=" + mapServletContext + ", mapServletPath=" + mapServletPath);

        Long externalInstituteId = -1L;
        try {
            externalInstituteId = Long.parseLong(externalInstituteIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("DataProviderDisplayInfoController.showExternalInstituteInfo(): cannot parse '" + externalInstituteIdStr + "', " + e.getMessage());
        }

        DataProviderDisplayInfo dataProviderDisplayInfo;
        if (externalInstituteId != -1) {
            dataProviderDisplayInfo = dataProviderDisplayInfoOperations.fromId(externalInstituteId, DataProviderDisplayInfoDAO.TYPE_NAME_EXTERNAL_INSTITUTE);
            searchLogger.addInstitutionInfo(dataProviderDisplayInfo);
        } else {
            dataProviderDisplayInfo = new DataProviderDisplayInfo();
        }

        ModelAndView modelAndView = new ModelAndView("externalInstituteDetails");
        modelAndView.addObject("dataProviderDisplayInfo", dataProviderDisplayInfo);

        return modelAndView;
    }

    @RequestMapping("/intezmeny/adatlap/{dataProviderId}")
    public ModelAndView showDataProviderInfo(HttpServletRequest httpServletRequest, @PathVariable("dataProviderId") String dataProviderIdStr, @RequestParam(value = "kisterseg", required = false) String subRegionIdStr, @RequestParam(value = "ful", required = false) String selectTab) throws Exception {
        String mapServletHost = PROTOCOL_HTTP + httpServletRequest.getHeader(REQUEST_HEADER_PARAM_HOST);
        String mapServletContext = httpServletRequest.getContextPath();
        String mapServletPath = "/mapservlet.kml";

        LOGGER.debug("mapServletHost=" + mapServletHost + ", mapServletContext=" + mapServletContext + ", mapServletPath=" + mapServletPath);

        Long dataProviderId = -1L;
        try {
            dataProviderId = Long.parseLong(dataProviderIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("DataProviderDisplayInfoController.showDataProviderInfo(): cannot parse '" + dataProviderIdStr + "' as dataProviderId, " + e.getMessage());
        }

        Long subRegionId = -1L;
        try {
            subRegionId = Long.parseLong(subRegionIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("DataProviderDisplayInfoController.showDataProviderInfo(): cannot parse '" + subRegionIdStr + "' as subRegionId, " + e.getMessage());
        }

        String pageParam = ServletRequestUtils.getStringParameter(httpServletRequest, "pageParam", "p");
        int page = ServletRequestUtils.getIntParameter(httpServletRequest, "page", 1);
        String pattern = ServletRequestUtils.getStringParameter(httpServletRequest, "pattern", null);
        String[] letters = ServletRequestUtils.getStringParameters(httpServletRequest, "letters");
        String backURI = ServletRequestUtils.getStringParameter(httpServletRequest, "backuri", (letters == null ? "institution.html" : "institutionlist.html"));

        DataProviderDisplayInfo dataProviderDisplayInfo;
        if (dataProviderId != -1) {
            dataProviderDisplayInfo = dataProviderDisplayInfoOperations.fromId(dataProviderId, DataProviderDisplayInfoDAO.TYPE_NAME_ACTIVE_DATAPROVIDER);
            searchLogger.addInstitutionInfo(dataProviderDisplayInfo);
        } else {
            dataProviderDisplayInfo = new DataProviderDisplayInfo();
        }

        dataProviderDisplayInfo.setBackURI(backURI);
        dataProviderDisplayInfo.setBackPageParam(pageParam);
        dataProviderDisplayInfo.setBackPage(page);
        dataProviderDisplayInfo.setBackPattern(pattern);
        dataProviderDisplayInfo.getLetters().addAll(Arrays.asList(letters));

        ModelAndView modelAndView = new ModelAndView("institutionDetails");
        modelAndView.addObject("dataProviderDisplayInfo", dataProviderDisplayInfo);

        modelAndView.addObject("showMap", true);
        if (subRegionId != -1L) {
            modelAndView.addObject("subRegionId", subRegionId);
        }

        DataProvider dataProvider;
        if (dataProviderId != -1) {
            dataProvider = dataProviderOperations.getDataProvider(dataProviderId);
        } else {
            dataProvider = null;
        }

        SubRegion subRegion = dataProviderOperations.getSubRegion(subRegionId);
        LOGGER.debug("subRegionId=" + subRegionId + ", subRegion=" + subRegion);

        if (selectTab == null || "".equals(selectTab.trim())) {
            selectTab = subRegion == null ? "adatgazda" : "kisterseg";
        }

        boolean hasDistrict = false;

        boolean showDataProviderTab = subRegion == null;
        boolean showDistrictOrSettlementTab = subRegion == null;

        //boolean dataProviderTabSelected = selectTab.equalsIgnoreCase("dataprovider");
        boolean dataProviderTabSelected = selectTab.equalsIgnoreCase("adatgazda");
        String selectDataProviderTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "adatgazda", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        boolean districtTabSelected = selectTab.equalsIgnoreCase("kerulet");
        String selectDistrictTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "kerulet", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        boolean settlementTabSelected = selectTab.equalsIgnoreCase("telepules");
        String selectSettlementTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "telepules", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        boolean subRegionTabSelected = selectTab.equalsIgnoreCase("kisterseg");
        String selectSubRegionTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "kisterseg", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        boolean countyTabSelected = selectTab.equalsIgnoreCase("megye");
        String selectCountyTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "megye", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        boolean regionTabSelected = selectTab.equalsIgnoreCase("regio");
        String selectRegionTabUrl = getSelectTabUrl(mapServletHost, mapServletContext, "regio", pageParam, page, pattern, letters, backURI, dataProviderId, subRegionId);

        Address address = dataProviderOperations.getAddress(dataProviderId);
        LOGGER.debug("address=" + address);

        String dataProviderMapUrl = null;
        String newAddress = null;

        GMIFrameSrcParam dataProviderFrame = null;
        GMIFrameSrcParam districtFrame = null;
        GMIFrameSrcParam settlementFrame = null;
        GMIFrameSrcParam subRegionFrame = null;
        GMIFrameSrcParam countyFrame = null;
        GMIFrameSrcParam regionFrame = null;

        if (subRegion != null || address != null) {

            if (address != null) {
                StringBuilder newAddressStr = new StringBuilder();
                newAddressStr.append(address.getZipCode().getZipCode()).append(" ");
                newAddressStr.append(address.getZipCode().getSettlement().getName());
                if (address.getAddress() != null && !"".equals(address.getAddress().trim())) {
                    newAddressStr.append(", ");
                    newAddressStr.append(address.getAddress());
                }
                newAddress = newAddressStr.toString();

                hasDistrict = address.getZipCode().getSettlement().getSubRegion().getId() == DataProviderMapConstants.CAPITAL_SUBREGION_ID;

                dataProviderMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.DataProvider);
                dataProviderFrame = GMIFrameSrcParam.getEInfoSzabDefault();
                dataProviderFrame.setNecessaryValues(dataProviderMapUrl, address.getLatitude(), address.getLongitude(), DataProviderMapConstants.DATAPROVIDER_ZOOM_FACTOR);

                String districtMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.District);
                districtFrame = GMIFrameSrcParam.getEInfoSzabDefault();
                districtFrame.setNecessaryValues(districtMapUrl, address.getZipCode().getSettlement().getLatitude(), address.getZipCode().getSettlement().getLongitude(), DataProviderMapConstants.DISTRICT_ZOOM_FACTOR);

                String settlementMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.Settlement);
                settlementFrame = GMIFrameSrcParam.getEInfoSzabDefault();
                settlementFrame.setNecessaryValues(settlementMapUrl, address.getZipCode().getSettlement().getLatitude(), address.getZipCode().getSettlement().getLongitude(), DataProviderMapConstants.SETTLEMENT_ZOOM_FACTOR);
            }

            String subRegionMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.SubRegion);
            subRegionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                subRegionFrame.setNecessaryValues(subRegionMapUrl, subRegion.getLatitude(), subRegion.getLongitude(), DataProviderMapConstants.SUBREGION_ZOOM_FACTOR);
            } else {
                subRegionFrame.setNecessaryValues(subRegionMapUrl, address.getZipCode().getSettlement().getSubRegion().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getLongitude(), DataProviderMapConstants.SUBREGION_ZOOM_FACTOR);
            }

            String countyMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.County);
            countyFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                countyFrame.setNecessaryValues(countyMapUrl, subRegion.getCounty().getLatitude(), subRegion.getCounty().getLongitude(), DataProviderMapConstants.COUNTY_ZOOM_FACTOR);
            } else {
                countyFrame.setNecessaryValues(countyMapUrl, address.getZipCode().getSettlement().getSubRegion().getCounty().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getCounty().getLongitude(), DataProviderMapConstants.COUNTY_ZOOM_FACTOR);
            }

            String regionMapUrl = getMapUrl(mapServletHost, mapServletContext, mapServletPath, dataProviderId, subRegionId, MapLayout.Region);
            regionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                regionFrame.setNecessaryValues(regionMapUrl, subRegion.getCounty().getRegion().getLatitude(), subRegion.getCounty().getRegion().getLongitude(), DataProviderMapConstants.REGION_ZOOM_FACTOR);
            } else {
                regionFrame.setNecessaryValues(regionMapUrl, address.getZipCode().getSettlement().getSubRegion().getCounty().getRegion().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getCounty().getRegion().getLongitude(), DataProviderMapConstants.REGION_ZOOM_FACTOR);
            }

        }
        modelAndView.addObject("googleMapsAPIKey", getGoogleMapsAPIKey(httpServletRequest));

        modelAndView.addObject("hasDistrict", hasDistrict);

        modelAndView.addObject("instituteName", dataProvider == null ? "" : dataProvider.getInstitution().getName());
        modelAndView.addObject("newAddress", newAddress);

        modelAndView.addObject("subRegionName", subRegion == null ? "" : subRegion.getName());
        modelAndView.addObject("subRegionId", subRegionId);

        modelAndView.addObject("showDataProviderTab", showDataProviderTab);
        modelAndView.addObject("showDistrictOrSettlementTab", showDistrictOrSettlementTab);

        modelAndView.addObject("dataProviderTabSelected", dataProviderTabSelected);
        modelAndView.addObject("selectDataProviderTabUrl", selectDataProviderTabUrl);

        modelAndView.addObject("districtTabSelected", districtTabSelected);
        modelAndView.addObject("selectDistrictTabUrl", selectDistrictTabUrl);

        modelAndView.addObject("settlementTabSelected", settlementTabSelected);
        modelAndView.addObject("selectSettlementTabUrl", selectSettlementTabUrl);

        modelAndView.addObject("subRegionTabSelected", subRegionTabSelected);
        modelAndView.addObject("selectSubRegionTabUrl", selectSubRegionTabUrl);

        modelAndView.addObject("countyTabSelected", countyTabSelected);
        modelAndView.addObject("selectCountyTabUrl", selectCountyTabUrl);

        modelAndView.addObject("regionTabSelected", regionTabSelected);
        modelAndView.addObject("selectRegionTabUrl", selectRegionTabUrl);

        modelAndView.addObject("dataProviderMapUrl", dataProviderMapUrl);
        modelAndView.addObject("dataProviderZoomFactor", DataProviderMapConstants.DATAPROVIDER_ZOOM_FACTOR);

        if (districtTabSelected) {
            modelAndView.addObject("displayFrame", districtFrame);
        } else if (settlementTabSelected) {
            modelAndView.addObject("displayFrame", settlementFrame);
        } else if (subRegionTabSelected) {
            modelAndView.addObject("displayFrame", subRegionFrame);
        } else if (countyTabSelected) {
            modelAndView.addObject("displayFrame", countyFrame);
        } else if (regionTabSelected) {
            modelAndView.addObject("displayFrame", regionFrame);
        } else {
            modelAndView.addObject("displayFrame", dataProviderFrame);
        }

        return modelAndView;
    }

    private String getSelectTabUrl(String mapServletHost, String mapServletContext, String tabName, String pageParam, int page, String pattern, String[] letters, String backuri, long dataProviderId, long subRegionId) {
        StringBuilder selectTabUrlBuilder = new StringBuilder(mapServletHost);
        selectTabUrlBuilder.append(mapServletContext);
        selectTabUrlBuilder.append("/").append("intezmeny/adatlap/").append(dataProviderId);
        selectTabUrlBuilder.append("?").append("ful").append("=").append(tabName);
        if (subRegionId != -1L) {
            selectTabUrlBuilder.append("&").append("kisterseg").append("=").append(subRegionId);
        }

        return selectTabUrlBuilder.toString();
    }

    private String getMapUrl(String mapServletHost, String mapServletContext, String mapServletPath, long dataProviderId, long subRegionId, MapLayout mapLayout) {
        StringBuilder mapUrlBuilder = new StringBuilder(mapServletHost);
        mapUrlBuilder.append(mapServletContext);
        mapUrlBuilder.append(mapServletPath);
        mapUrlBuilder.append("?");
        mapUrlBuilder.append("dataProviderId").append("=").append(dataProviderId);
        if (subRegionId != -1L) {
            mapUrlBuilder.append("&");
            mapUrlBuilder.append("subRegionId").append("=").append(subRegionId);
        }
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("layout").append("=").append(mapLayout);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("uid").append("=").append(System.currentTimeMillis());

        return mapUrlBuilder.toString();
    }

    @Deprecated
    private String encodeUrl(String url) {
        String encodedUrl = null;
        if (url != null) {
            try {
                encodedUrl = URLEncoder.encode(url, URL_ENCODING);
            } catch (UnsupportedEncodingException e) {
                LOGGER.error("Error during URL encode, using " + URL_ENCODING + ": " + e.getMessage(), e);
            }
        }
        return encodedUrl;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }
}
