/**
 * Created by IntelliJ IDEA.
 * User: tnagy
 * Date: 2009.12.18.
 * Time: 15:25:34
 */
package hu.kozadat.search.presentation.gmaps;

import hu.kozadat.search.persistence.DataProviderOperations;
import hu.kozadat.search.presentation.DataProviderMapConstants;
import hu.kozadat.search.presentation.GMIFrameSrcParam;
import hu.kozadat.search.service.SearchUIConfiguration;
import hu.kozadat.search.web.map.MapLayout;
import hu.scriptum.einfoszab.datamodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.ServletRequestUtils;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Controller
public class DataProviderMapsController {

    private static final Logger LOGGER = LoggerFactory.getLogger(DataProviderMapsController.class);

    private SearchUIConfiguration searchUIConfiguration;
    private DataProviderOperations dataProviderOperations;

    private String mapServletPath = "/mapservlet.kml";

    @RequestMapping("/regio/{regionId}")
    public ModelAndView showRegionMap(@PathVariable("regionId") String regionIdStr) {

        Long regionId = null;
        try {
            regionId = Long.parseLong(regionIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("showRegionMap(): cannot parse '" + regionIdStr + "' as Long, " + e.getMessage());
        }

        if (regionId == null) {
            return new ModelAndView("redirect:/intezmeny");
        }

        Region region = dataProviderOperations.getRegion(regionId);

        boolean subRegionTabSelected = false;
        boolean countyTabSelected = false;
        boolean regionTabSelected = true;

        String viewName = "regionSearch";
        ModelAndView modelAndView = new ModelAndView(viewName);

        GMIFrameSrcParam regionFrame;

        regionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
        String regionMapUrl = getRegionMapUrl(regionId);
        regionFrame.setNecessaryValues(regionMapUrl, region.getLatitude(), region.getLongitude(), DataProviderMapConstants.REGION_ZOOM_FACTOR);

        modelAndView.addObject("subRegionTabSelected", subRegionTabSelected);
        modelAndView.addObject("countyTabSelected", countyTabSelected);
        modelAndView.addObject("regionTabSelected", regionTabSelected);

        modelAndView.addObject("displayFrame", regionFrame);

        return modelAndView;
    }

    @RequestMapping("/megye/{countyId}")
    public ModelAndView showCountyMap(@PathVariable("countyId") String countyIdStr) {

        Long countyId = null;
        try {
            countyId = Long.parseLong(countyIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("showCountyMap(): cannot parse '" + countyIdStr + "' as Long, " + e.getMessage());
        }

        if (countyId == null) {
            return new ModelAndView("redirect:/intezmeny");
        }

        County county = dataProviderOperations.getCounty(countyId);

        boolean subRegionTabSelected = false;
        boolean countyTabSelected = true;
        boolean regionTabSelected = false;

        String viewName = "regionSearch";
        ModelAndView modelAndView = new ModelAndView(viewName);

        GMIFrameSrcParam countyFrame;

        countyFrame = GMIFrameSrcParam.getEInfoSzabDefault();
        String countyMapUrl = getCountyMapUrl(countyId);
        countyFrame.setNecessaryValues(countyMapUrl, county.getLatitude(), county.getLongitude(), DataProviderMapConstants.COUNTY_ZOOM_FACTOR);

        modelAndView.addObject("subRegionTabSelected", subRegionTabSelected);
        modelAndView.addObject("countyTabSelected", countyTabSelected);
        modelAndView.addObject("regionTabSelected", regionTabSelected);

        modelAndView.addObject("selectedCounty", county);

        modelAndView.addObject("displayFrame", countyFrame);

        return modelAndView;

    }

    @RequestMapping("/kisterseg/{subRegionId}")
    public ModelAndView showSubRegionMap(@PathVariable("subRegionId") String subRegionIdStr) {

        Long subRegionId = null;
        try {
            subRegionId = Long.parseLong(subRegionIdStr);
        } catch (NumberFormatException e) {
            LOGGER.warn("showCountyMap(): cannot parse '" + subRegionIdStr + "' as Long, " + e.getMessage());
        }

        if (subRegionId == null) {
            return new ModelAndView("redirect:/intezmeny");
        }

        SubRegion subRegion = dataProviderOperations.getSubRegion(subRegionId);

        if (subRegion == null) {
            LOGGER.warn("SubRegion not found with id=" + subRegionId);
            return new ModelAndView("redirect:/intezmeny");
        }

        boolean subRegionTabSelected = true;
        boolean countyTabSelected = false;
        boolean regionTabSelected = false;

        String viewName = "regionSearch";
        ModelAndView modelAndView = new ModelAndView(viewName);

        GMIFrameSrcParam subRegionFrame;

        subRegionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
        String subRegionMapUrl = getSubRegionMapUrl(subRegionId);
        subRegionFrame.setNecessaryValues(subRegionMapUrl, subRegion.getLatitude(), subRegion.getLongitude(), DataProviderMapConstants.SUBREGION_ZOOM_FACTOR);

        modelAndView.addObject("subRegionTabSelected", subRegionTabSelected);
        modelAndView.addObject("countyTabSelected", countyTabSelected);
        modelAndView.addObject("regionTabSelected", regionTabSelected);
        modelAndView.addObject("selectedSubRegion", subRegion);

        modelAndView.addObject("displayFrame", subRegionFrame);

        return modelAndView;

    }

    @Deprecated
    protected ModelAndView handleRequestInternal(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse) throws Exception {

        ModelAndView modelAndView = new ModelAndView("maps");

        long dataProviderId = ServletRequestUtils.getLongParameter(httpServletRequest, "dataProviderId", -1L);
        DataProvider dataProvider = dataProviderOperations.getDataProvider(dataProviderId);
        LOGGER.debug("dataProviderId=" + dataProviderId + ", dataProvider=" + dataProvider);

        long subRegionId = ServletRequestUtils.getLongParameter(httpServletRequest, "subRegionId", -1L);
        SubRegion subRegion = dataProviderOperations.getSubRegion(subRegionId);
        LOGGER.debug("subRegionId=" + subRegionId + ", subRegion=" + subRegion);

        boolean hasDistrict = false;

        boolean showDataProviderTab = subRegion == null;
        boolean showDistrictOrSettlementTab = subRegion == null;

        boolean dataProviderTabSelected = true;
        boolean districtTabSelected = false;
        boolean settlementTabSelected = false;
        boolean subRegionTabSelected = false;
        boolean countyTabSelected = false;
        boolean regionTabSelected = false;

        Address address = dataProviderOperations.getAddress(dataProviderId);
        LOGGER.debug("address=" + address);

        if (subRegion != null || address != null) {
            String dataProviderMapUrl = null;
            String newAddress = null;

            GMIFrameSrcParam districtFrame = null;
            GMIFrameSrcParam settlementFrame = null;
            GMIFrameSrcParam subRegionFrame;
            GMIFrameSrcParam countyFrame;
            GMIFrameSrcParam regionFrame;

            if (address != null) {
                StringBuilder newAddressStr = new StringBuilder();
                newAddressStr.append(address.getZipCode().getZipCode()).append(" ");
                newAddressStr.append(address.getZipCode().getSettlement().getName()).append(", ");
                newAddressStr.append(address.getAddress());
                newAddress = newAddressStr.toString();

                hasDistrict = address.getZipCode().getSettlement().getSubRegion().getId() == DataProviderMapConstants.CAPITAL_SUBREGION_ID;

                dataProviderMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.DataProvider);

                String districtMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.District);
                districtFrame = GMIFrameSrcParam.getEInfoSzabDefault();
                districtFrame.setNecessaryValues(districtMapUrl, address.getZipCode().getSettlement().getLatitude(), address.getZipCode().getSettlement().getLongitude(), DataProviderMapConstants.DISTRICT_ZOOM_FACTOR);

                String settlementMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.Settlement);
                settlementFrame = GMIFrameSrcParam.getEInfoSzabDefault();
                settlementFrame.setNecessaryValues(settlementMapUrl, address.getZipCode().getSettlement().getLatitude(), address.getZipCode().getSettlement().getLongitude(), DataProviderMapConstants.SETTLEMENT_ZOOM_FACTOR);
            }

            String subRegionMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.SubRegion);
            subRegionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                subRegionFrame.setNecessaryValues(subRegionMapUrl, subRegion.getLatitude(), subRegion.getLongitude(), DataProviderMapConstants.SUBREGION_ZOOM_FACTOR);
            } else {
                subRegionFrame.setNecessaryValues(subRegionMapUrl, address.getZipCode().getSettlement().getSubRegion().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getLongitude(), DataProviderMapConstants.SUBREGION_ZOOM_FACTOR);
            }

            String countyMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.County);
            countyFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                countyFrame.setNecessaryValues(countyMapUrl, subRegion.getCounty().getLatitude(), subRegion.getCounty().getLongitude(), DataProviderMapConstants.COUNTY_ZOOM_FACTOR);
            } else {
                countyFrame.setNecessaryValues(countyMapUrl, address.getZipCode().getSettlement().getSubRegion().getCounty().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getCounty().getLongitude(), DataProviderMapConstants.COUNTY_ZOOM_FACTOR);
            }

            String regionMapUrl = getMapUrl(dataProviderId, subRegionId, MapLayout.Region);
            regionFrame = GMIFrameSrcParam.getEInfoSzabDefault();
            if (subRegion != null) {
                regionFrame.setNecessaryValues(regionMapUrl, subRegion.getCounty().getRegion().getLatitude(), subRegion.getCounty().getRegion().getLongitude(), DataProviderMapConstants.REGION_ZOOM_FACTOR);
            } else {
                regionFrame.setNecessaryValues(regionMapUrl, address.getZipCode().getSettlement().getSubRegion().getCounty().getRegion().getLatitude(), address.getZipCode().getSettlement().getSubRegion().getCounty().getRegion().getLongitude(), DataProviderMapConstants.REGION_ZOOM_FACTOR);
            }

            modelAndView.addObject("hasDistrict", hasDistrict);

            modelAndView.addObject("instituteName", dataProvider.getInstitution().getName());
            modelAndView.addObject("newAddress", newAddress);

            modelAndView.addObject("subRegionName", subRegion == null ? "" : subRegion.getName());
            modelAndView.addObject("subRegionId", subRegionId);

            modelAndView.addObject("showDataProviderTab", showDataProviderTab);
            modelAndView.addObject("showDistrictOrSettlementTab", showDistrictOrSettlementTab);

            modelAndView.addObject("dataProviderTabSelected", dataProviderTabSelected);
            modelAndView.addObject("districtTabSelected", districtTabSelected);
            modelAndView.addObject("settlementTabSelected", settlementTabSelected);
            modelAndView.addObject("subRegionTabSelected", subRegionTabSelected);
            modelAndView.addObject("countyTabSelected", countyTabSelected);
            modelAndView.addObject("regionTabSelected", regionTabSelected);

            modelAndView.addObject("dataProviderMapUrl", dataProviderMapUrl);
            modelAndView.addObject("dataProviderZoomFactor", DataProviderMapConstants.DATAPROVIDER_ZOOM_FACTOR);

            modelAndView.addObject("districtFrame", districtFrame);

            modelAndView.addObject("settlementFrame", settlementFrame);

            modelAndView.addObject("subRegionFrame", subRegionFrame);

            modelAndView.addObject("countyFrame", countyFrame);

            modelAndView.addObject("regionFrame", regionFrame);
        }

        LOGGER.debug("handleRequestInternal() FINISHED");
        return modelAndView;
    }

    private String getCoords(Double latitude, Double longitude) {
        StringBuilder builder = new StringBuilder();
        if (latitude != null && longitude != null) {
            builder.append(latitude).append(",").append(longitude);
        }
        return builder.toString();
    }

    private String getMapUrl(long dataProviderId, long subRegionId, MapLayout mapLayout) {
        String host = searchUIConfiguration.getApplicationHost();
        String context = searchUIConfiguration.getApplicationContext();

        StringBuilder mapUrlBuilder = new StringBuilder(host);
        mapUrlBuilder.append(context);
        mapUrlBuilder.append(mapServletPath);
        mapUrlBuilder.append("?");
        mapUrlBuilder.append("dataProviderId").append("=").append(dataProviderId);
        if (subRegionId != -1L) {
            mapUrlBuilder.append("&");
            mapUrlBuilder.append("subRegionId").append("=").append(subRegionId);
        }
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("layout").append("=").append(mapLayout);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("uid").append("=").append(System.currentTimeMillis());

        String mapUrl = mapUrlBuilder.toString();

        return mapUrl;
    }

    private String getRegionMapUrl(long regionId) {
        String host = searchUIConfiguration.getApplicationHost();
        String context = searchUIConfiguration.getApplicationContext();

        StringBuilder mapUrlBuilder = new StringBuilder(host);
        mapUrlBuilder.append(context);
        mapUrlBuilder.append(mapServletPath);
        mapUrlBuilder.append("?");
        mapUrlBuilder.append("regionId").append("=").append(regionId);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("layout").append("=").append(MapLayout.Region);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("uid").append("=").append(System.currentTimeMillis());

        String mapUrl = mapUrlBuilder.toString();

        return mapUrl;
    }

    private String getCountyMapUrl(long countyId) {
        String host = searchUIConfiguration.getApplicationHost();
        String context = searchUIConfiguration.getApplicationContext();

        StringBuilder mapUrlBuilder = new StringBuilder(host);
        mapUrlBuilder.append(context);
        mapUrlBuilder.append(mapServletPath);
        mapUrlBuilder.append("?");
        mapUrlBuilder.append("countyId").append("=").append(countyId);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("layout").append("=").append(MapLayout.County);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("uid").append("=").append(System.currentTimeMillis());

        String mapUrl = mapUrlBuilder.toString();

        return mapUrl;
    }

    private String getSubRegionMapUrl(long subRegionId) {
        String host = searchUIConfiguration.getApplicationHost();
        String context = searchUIConfiguration.getApplicationContext();

        StringBuilder mapUrlBuilder = new StringBuilder(host);
        mapUrlBuilder.append(context);
        mapUrlBuilder.append(mapServletPath);
        mapUrlBuilder.append("?");
        mapUrlBuilder.append("subRegionId").append("=").append(subRegionId);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("layout").append("=").append(MapLayout.SubRegion);
        mapUrlBuilder.append("&");
        mapUrlBuilder.append("uid").append("=").append(System.currentTimeMillis());

        String mapUrl = mapUrlBuilder.toString();

        return mapUrl;
    }

    @Autowired
    public void setDataProviderOperations(DataProviderOperations dataProviderOperations) {
        this.dataProviderOperations = dataProviderOperations;
    }

    @Autowired
    public void setSearchUIConfiguration(SearchUIConfiguration searchUIConfiguration) {
        this.searchUIConfiguration = searchUIConfiguration;
    }

}
