package hu.kozadat.search.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.servlet.HandlerExceptionResolver;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

public class KozadatExceptionResolver implements HandlerExceptionResolver {

    private static final Logger LOGGER = LoggerFactory.getLogger(KozadatExceptionResolver.class);

    private static final int MAX_MESSAGE_LENGTH = 120;
    private static final int DETAILS_INITIAL_SIZE = 4096;
    private static final String TRUNCATE_POSTFIX = "...";
    private static final int DETAILS_MAX_LENGTH = 2048;
    private static final String ATTR_NAME_HANDLED_EXCEPTION = "handledException";

    private String errorView;
    private int errorCode;
    private String moduleName;
    private JdbcTemplate jdbcTemplate;
    private static final SimpleDateFormat YYYYMMDDHHMMSS_FORMATTER = new SimpleDateFormat("yyyyMMddHHmmss");

    private static String formatDate(Date date) {
        synchronized (YYYYMMDDHHMMSS_FORMATTER) {
            return YYYYMMDDHHMMSS_FORMATTER.format(date);
        }
    }

    public void setJdbcTemplate(JdbcTemplate jdbcTemplate) {
        this.jdbcTemplate = jdbcTemplate;
    }

    public void setErrorCode(int errorCode) {
        this.errorCode = errorCode;
    }

    public void setErrorView(String errorView) {
        this.errorView = errorView;
    }

    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
    }

    public ModelAndView resolveException(HttpServletRequest request, HttpServletResponse response, Object handler, Exception exception) {


        ModelAndView modelAndView = new ModelAndView(errorView);
        response.setStatus(errorCode);

        try {

            if (exception != null) {

                LOGGER.error("resolveException(): ERROR: {}", exception.getMessage(), exception);

                String message = exception.getMessage();
                if (message != null && message.length() > MAX_MESSAGE_LENGTH) {
                    message = message.substring(0, MAX_MESSAGE_LENGTH - TRUNCATE_POSTFIX.length() - 1) + TRUNCATE_POSTFIX;
                }

                StringWriter writer = new StringWriter(DETAILS_INITIAL_SIZE);
                exception.printStackTrace(new PrintWriter(writer));
                String details = writer.toString();
                if (details.length() > DETAILS_MAX_LENGTH) {
                    details = details.substring(0, DETAILS_MAX_LENGTH - TRUNCATE_POSTFIX.length() - 1) + TRUNCATE_POSTFIX;
                }

                String date = formatDate(new Date());

                String exceptionClass = exception.getClass().getName();

                jdbcTemplate.update(
                        "INSERT INTO system_errors(occure_time, module, title, java_exception, exception_class) VALUES (?, ?, ?, ?, ?)",
                        new Object[]{date, moduleName, message, details, exceptionClass}
                );
            }

            modelAndView.addObject(ATTR_NAME_HANDLED_EXCEPTION, exception);

        } catch (Exception e) {
            LOGGER.error("resolveException(): GENERAL ERROR: {}", e.getMessage(), e);
            modelAndView.addObject(ATTR_NAME_HANDLED_EXCEPTION, e);
        }

        return modelAndView;

    }
}
