package hu.kozadat.search.ws;

import hu.kozadat.search.model.DataProviderDisplayInfo;
import hu.kozadat.search.persistence.core.DataProviderDisplayInfoDAO;
import hu.kozadat.search.persistence.core.RdbmsDataProviderOperations;
import hu.scriptum.einfoszab.datamodel.Schema;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import javax.jws.WebService;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.WebServiceContext;
import javax.xml.ws.handler.MessageContext;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: codex
 * Date: May 5, 2008
 * Time: 11:02:51 PM
 * To change this template use File | Settings | File Templates.
 */
@WebService(serviceName = "OAISearchService", endpointInterface = "hu.kozadat.search.ws.SearchSEI", targetNamespace = "http://ws.textsearch.kozadat.hu")
public class SearchService implements SearchSEI {

    public static final Logger LOGGER = LoggerFactory.getLogger(SearchService.class);

    @Resource
    private WebServiceContext wsContext;

    @PostConstruct
    public void init() {
        LOGGER.info("SearchService created.");
    }

    @PreDestroy
    public void release() {
        LOGGER.debug("SearchService destroyed.");
    }

    public Collection<SearchResultItem> search(String query, Integer schema) {
        ServletContext servletContext = (ServletContext) wsContext.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
        WebApplicationContext webContext = WebApplicationContextUtils.getWebApplicationContext(servletContext);

        Collection<SearchResultItem> searchResultItems;

        WSSimpleSearch wsSimpleSearch = (WSSimpleSearch) webContext.getBean("wsSimpleSearch");
        searchResultItems = wsSimpleSearch.search(query, schema);

        return searchResultItems;
    }

    public Collection<SchemaItem> getSchemas() {
        ServletContext servletContext = (ServletContext) wsContext.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
        WebApplicationContext webContext = WebApplicationContextUtils.getWebApplicationContext(servletContext);
        Collection<SchemaItem> res = new ArrayList<SchemaItem>();

        // RdbmsDataProviderOperations ops = (RdbmsDataProviderOperations) webContext.getBean("dpOperations");
        RdbmsDataProviderOperations ops = (RdbmsDataProviderOperations) webContext.getBean("dataProviderOperations");
        List schemas = ops.getSupportedSchemas();

        for (Object actObj : schemas) {
            Schema actSchema = (Schema) actObj;
            res.add(new SchemaItem(actSchema.getId(), actSchema.getName()));
        }

        return res;
    }

    public DataProviderWSInfo getDataProvider(int id) throws NoSuchDataProviderException {
        ServletContext servletContext = (ServletContext) wsContext.getMessageContext().get(MessageContext.SERVLET_CONTEXT);
        WebApplicationContext webContext = WebApplicationContextUtils.getWebApplicationContext(servletContext);

        JdbcTemplate jdbcTemplate = (JdbcTemplate) webContext.getBean("jdbcTemplate");
        DataProviderDisplayInfo resDisp = new DataProviderDisplayInfoDAO().fromId(jdbcTemplate, id);
        if (resDisp == null) {
            throw new NoSuchDataProviderException(NoSuchDataProviderException.NO_SUCH_DATA_PROVIDER_ID_ERROR);
        }

        DataProviderWSInfo res = new DataProviderWSInfo();
        res.setAddress(resDisp.getAddress());
        res.setEmail(resDisp.getEmail());
        res.setFaxNumber(resDisp.getFaxNumber());
        res.setInstitutionShortName(resDisp.getInstitutionShortName());
        res.setLeader(resDisp.getLeader());
        res.setPhoneNumber(resDisp.getPhoneNumber());
        res.setPostalAddress(resDisp.getPostalAddress());
        res.setWebsite(resDisp.getWebsite());
        res.setUserType(resDisp.getUserType());

        return res;
    }

    public boolean hasResult(String searchExpression) {
        return true;
    }

    public int getResultCount(String searchExpression) {
        return 1;
    }
}
